@@ -20,7 +20,7 @@
 
  */
 void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, const unsigned int expert);
-int fat1x_boot_sector(t_param_disk *disk_car, t_diskext *partition, const int debug, const int dump_ind, const unsigned int expert);
-int fat32_boot_sector(t_param_disk *disk_car, t_diskext *partition, const int debug, const int dump_ind, const unsigned int expert);
-int ntfs_boot_sector(t_param_disk *disk_car, t_diskext *partition, const int debug, const int dump_ind);
+int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert);
+int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert);
+int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind);
 
