@@ -25,29 +25,37 @@
  
 #include <stdarg.h>
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include <ctype.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>	/* geteuid */
+#endif
 #include "types.h"
 #include "common.h"
 #include "lang.h"
-#include "intrface.h"
+#include "intrf.h"
 #include "fnctdsk.h"
+#include "chgtype.h"
 
-extern char* nom_os[256];
-int change_part_type(const t_param_disk *disk_car,t_diskext *partition)
+int change_part_type(const t_param_disk *disk_car,t_partition *partition)
 {
   char response[100];
   int nbr=0;
   int i;
   aff_buffer(BUFFER_RESET,"Q");
   aff_buffer(BUFFER_ADD,"List of partition type\n");
-  for(i=0;i<=255;i++)
+  for(i=0;i<=0xFF;i++)
   {
-    if(nom_os[i]!=NULL && strcmp(nom_os[i],"reserved")!=0)
+    const char *part_name;
+    part_name=partition_type(i, disk_car->arch->parttype_name_table);
+    if(part_name!=NULL)
     {
-      aff_buffer(BUFFER_ADD,"%02x %-20s ",i,nom_os[i]);
+      aff_buffer(BUFFER_ADD,"%02x %-20s ",i,part_name);
       nbr++;
       if(nbr==3)
       {
@@ -58,14 +66,16 @@ int change_part_type(const t_param_disk *disk_car,t_diskext *partition)
   }
   aff_copy(stdscr);
   wmove(stdscr,4,0);
-  aff_part(stdscr,AFF_PART_NONL,disk_car,partition);
+  aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
   aff_buffer(BUFFER_DISPLAY,"Q",stdscr);
   wmove(stdscr,23,0);
   wdoprintf(stdscr,"New partition type [current %02x] ?",partition->part_type);
   if (get_string(response, sizeof(response), NULL) > 0) {
     int tmp_val = strtol(response, NULL, 16);
-    if ((tmp_val > 0) && (tmp_val <= 255) && (is_extended(tmp_val)==0))
+    if(disk_car->arch->is_part_type_ok(tmp_val)!=0)
+    {
       partition->part_type = tmp_val;
+    }
   }
   return 0;
 }
