@@ -25,24 +25,31 @@
  
 #include <stdarg.h>
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include <ctype.h>
-#include <unistd.h>	/* geteuid */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "lang.h"
-#include "intrface.h"
+#include "intrf.h"
 #include "fnctdsk.h"
+#include "edit.h"
 
-static void interface_editor_position(const t_param_disk *disk_car, unsigned long int *lba);
+static void interface_editor_position(const t_param_disk *disk_car, uint64_t *lba);
 static int dump_editor(const unsigned char *nom_dump,const unsigned int lng, const int menu_pos);
 
 void interface_editor(t_param_disk *disk_car)
 {
   int done = FALSE;
-  dword hd_offset=0;
-  ecrit_rapport("%s",disk_car->description(disk_car));
+  uint64_t hd_offset=0;
+  ecrit_rapport("%s\n",disk_car->description(disk_car));
   while (done==FALSE)
   {
     static struct MenuItem menuEditor[]=
@@ -59,30 +66,30 @@ void interface_editor(t_param_disk *disk_car)
 	break;
       case 'D':
 	{
-	  unsigned char buffer[SECTOR_SIZE];
+	  unsigned char buffer[disk_car->sector_size];
 	  int menu_pos=MENU_DOWN;
 	  while(done==0)
 	  {
 	    wmove(stdscr,4,0);
 	    wclrtoeol(stdscr);
-	    wdoprintf(stdscr,"%ld ", hd_offset);
-	    aff_LBA2CHS(disk_car,hd_offset);
-	    if(disk_car->read(disk_car,1, buffer, hd_offset))
+	    wdoprintf(stdscr,"%lu ", (unsigned long)(hd_offset/disk_car->sector_size));
+	    aff_LBA2CHS(disk_car,hd_offset/disk_car->sector_size);
+	    if(disk_car->read(disk_car,sizeof(buffer), buffer, hd_offset))
 	      wdoprintf(stdscr,msg_PART_RD_ERR);
 	    else
 	    {
-	      menu_pos=dump_editor(buffer,SECTOR_SIZE,menu_pos);
+	      menu_pos=dump_editor(buffer,disk_car->sector_size,menu_pos);
 	      switch(menu_pos)
 	      {
 		case MENU_UP:
 		  if(hd_offset>0)
-		    hd_offset--;
+		    hd_offset-=disk_car->sector_size;
 		  else
 		    menu_pos=MENU_DOWN;
 		  break;
 		case MENU_DOWN:
-		  if(hd_offset<disk_car->size-1)
-		    hd_offset++;
+		  if(hd_offset<disk_car->disk_size)
+		    hd_offset+=disk_car->sector_size;
 		  else
 		    menu_pos=MENU_UP;
 		  break;
@@ -103,7 +110,7 @@ void interface_editor(t_param_disk *disk_car)
   }
 }
 
-static void interface_editor_position(const t_param_disk *disk_car,unsigned long int *lba)
+static void interface_editor_position(const t_param_disk *disk_car,uint64_t *lba)
 {
   t_CHS position;
   int done = FALSE;
@@ -111,9 +118,9 @@ static void interface_editor_position(const t_param_disk *disk_car,unsigned long
   char response[LINE_LENGTH];
   unsigned int tmp_val;
   int command;
-  position.cylinder=LBA2cylinder(disk_car,*lba);
-  position.head=LBA2head(disk_car,*lba);
-  position.sector=LBA2sector(disk_car,*lba);
+  position.cylinder=offset2cylinder(disk_car,*lba);
+  position.head=offset2head(disk_car,*lba);
+  position.sector=offset2sector(disk_car,*lba);
   while (done==FALSE) {
 	static struct MenuItem menuGeometry[]=
 	{
@@ -168,7 +175,7 @@ static void interface_editor_position(const t_param_disk *disk_car,unsigned long
 		break;
 	}
   }
-  *lba=CHS2LBA(disk_car,&position);
+  *lba=CHS2offset(disk_car,&position);
 }
 
 static int dump_editor(const unsigned char *nom_dump,const unsigned int lng, const int menu_pos)
