@@ -25,13 +25,20 @@
  
 #include <stdarg.h>
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include <ctype.h>
-#include <unistd.h>	/* geteuid */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include "types.h"
 #include "common.h"
-#include "intrface.h"
+#include "intrf.h"
+#include "chgtype.h"
 
 void change_geometry(t_param_disk *disk_car)
 {
@@ -40,7 +47,7 @@ void change_geometry(t_param_disk *disk_car)
   char response[LINE_LENGTH];
   int tmp_val;
   int command;
-  int default_option=3;
+  int default_option=4;
   int modified=0;
 
   while (done==FALSE) {
@@ -49,12 +56,13 @@ void change_geometry(t_param_disk *disk_car)
 	  { 'c', "Cylinders", "Change cylinder geometry" },
 	  { 'h', "Heads", "Change head geometry" },
 	  { 's', "Sectors", "Change sector geometry" },
-	  { 'd', "Done", "Done with changing geometry" },
+	  { 'n', "Sector Size", "Change sector size (WARNING: VERY DANGEROUS!)" },
+	  { 'q', "Quit", "Done with changing geometry" },
 	  { 0, NULL, NULL }
 	};
-	aff_copy_full(stdscr);
+	aff_copy(stdscr);
 	wmove(stdscr,5,0);
-	wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+	wdoprintf(stdscr,"%s, sector size=%u\n",disk_car->description(disk_car),disk_car->sector_size);
 	wmove(stdscr,COMMAND_LINE_Y, COMMAND_LINE_X);
 	wclrtoeol(stdscr);
 	wrefresh(stdscr);
@@ -83,7 +91,7 @@ void change_geometry(t_param_disk *disk_car)
 		    disk_car->CHS.head = tmp_val-1;
 		    if(modified==0)
 		    {
-		      disk_car->CHS.cylinder=(disk_car->size/(disk_car->CHS.head+1))/disk_car->CHS.sector-1;
+		      disk_car->CHS.cylinder=(disk_car->disk_size/disk_car->sector_size/(disk_car->CHS.head+1))/disk_car->CHS.sector-1;
 		    }
 		  } else
 		    wdoprintf(stdscr,"Illegal heads value");
@@ -92,25 +100,46 @@ void change_geometry(t_param_disk *disk_car)
 		break;
 	  case 'S':
 		sprintf(def, "%u", disk_car->CHS.sector);
-		mvwaddstr(stdscr,COMMAND_LINE_Y, COMMAND_LINE_X, "Enter the number of sectors per track: ");
+		mvwaddstr(stdscr,COMMAND_LINE_Y, COMMAND_LINE_X, "Enter the number of sectors per track (1-63): ");
 		if (get_string(response, LINE_LENGTH, def) > 0) {
 		  tmp_val = atoi(response);
-		  if (tmp_val > 0 && tmp_val <= MAX_SECTORS) {
+		  /* SUN partition can have more than 63 sectors */
+		  if (tmp_val > 0) {
 			disk_car->CHS.sector = tmp_val;
+			if(modified==0)
+			{
+			  disk_car->CHS.cylinder=(disk_car->disk_size/disk_car->sector_size/(disk_car->CHS.head+1))/disk_car->CHS.sector-1;
+			}
 		  } else
 			wdoprintf(stdscr,"Illegal sectors value");
 		}
 		default_option=3;
-		modified=1;
+		break;
+	  case 'N':
+		sprintf(def, "%u", disk_car->sector_size);
+		mvwaddstr(stdscr,COMMAND_LINE_Y, COMMAND_LINE_X, "Enter the sector size (512, 1024, 2048, 4096): ");
+		if (get_string(response, LINE_LENGTH, def) > 0) {
+		  tmp_val = atoi(response);
+		  /* FIXME using 3*512=1536 as sector size and */
+		  /* 63/3=21 for number of sectors is an easy way to test */
+		  if (tmp_val==512 || tmp_val==1024 || tmp_val==2048 || tmp_val==4096 || tmp_val==3*512) {
+		    disk_car->sector_size = tmp_val;
+		    if(modified==0)
+		    {
+		      disk_car->CHS.cylinder=(disk_car->disk_size/disk_car->sector_size/(disk_car->CHS.head+1))/disk_car->CHS.sector-1;
+		    }
+		  } else
+		    wdoprintf(stdscr,"Illegal sector size");
+		}
+		default_option=4;
 		break;
 	  case key_ESC:
-	  case 'D':
-/*  	  case 'Q': */
+  	  case 'Q':
 		done = TRUE;
 		break;
 	}
   }
-  disk_car->size=(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector;
-  ecrit_rapport("New geometry\n%s", disk_car->description(disk_car));
+  disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+  ecrit_rapport("New geometry\n%s sector_size=%u\n", disk_car->description(disk_car), disk_car->sector_size);
 }
 
