@@ -24,8 +24,12 @@
 #endif
  
 #include <ctype.h>      /* toupper, tolower */
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -35,21 +39,18 @@
 #include "testdisk.h"
 #include "fat.h"
 #include "ext2.h"
+#include "intrf.h"
 #include "intrface.h"
 #include "md.h"
 #include "adv.h"
 #define RO 1
 #define RW 0
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part,const int align);
-static void init_structure(const t_param_disk *disk_car,t_list_part *list_part, const int debug);
-static void init_part_order(t_list_part *list_part);
 static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part);
 static t_list_part *reduce_structure(t_list_part *list_part);
 static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align,const int debug);
-static int write_mbr(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug);
-static int write_all_log(t_param_disk *disk_car, const t_list_part *list_part, const int ro , const int debug, const int align);
-static int diff(const unsigned char buffer[SECTOR_SIZE], const unsigned char buffer_org[SECTOR_SIZE]);
 static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert);
+static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part_bad);
 
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part, const int align)
 {
@@ -59,7 +60,7 @@ static void align_structure(const t_param_disk *disk_car, t_list_part *list_part
   for(element=list_part;element!=NULL;element=element->next)
   {
     t_CHS end;
-    LBA2CHS(disk_car,element->part->lba+element->part->part_size-1,&end);
+    offset2CHS(disk_car,element->part->part_offset+element->part->part_size-1,&end);
     if(align>0)
     {
       end.sector=disk_car->CHS.sector;
@@ -68,120 +69,10 @@ static void align_structure(const t_param_disk *disk_car, t_list_part *list_part
     {
       end.head=disk_car->CHS.head;
     }
-    element->part->part_size=CHS2LBA(disk_car, &end)-element->part->lba+1;
+    element->part->part_size=(uint64_t)CHS2offset(disk_car, &end)-element->part->part_offset+disk_car->sector_size;
   }
 }
 
-static void init_structure(const t_param_disk *disk_car,t_list_part *list_part, const int debug)
-{
-  /* Reconstruit une structure 
-   * nbr_prim nombre de partition primaire
-   * end_log_block : partition etendu du block etendu
-   * nbr_log_block : nombre de block de partition etendu
-   *                    devrait etre 0 ou 1 */
-  unsigned int nbr_prim=0, nbr_log_block=0;
-  t_list_part *element;
-  t_list_part *end_log_block=NULL;
-  t_list_part *end_biggest_log_block=NULL;
-  unsigned int log_block_size=0,biggest_log_block_size=0;
-  t_list_part *new_list_part=NULL;
-  /* Create new list */
-  for(element=list_part;element!=NULL;element=element->next)
-    element->to_be_removed=0;
-  for(element=list_part;element!=NULL;element=element->next)
-  {
-    int to_be_removed=0;
-    t_list_part *element2;
-    for(element2=element->next;element2!=NULL;element2=element2->next)
-      if(element->part->lba+element->part->part_size-1 >= element2->part->lba)
-      {
-	to_be_removed=1;
-	element2->to_be_removed=1;
-      }
-    if(to_be_removed)
-      element->to_be_removed=1;
-    if(element->to_be_removed==0)
-      new_list_part=insert_new_partition(new_list_part,element->part);
-  }
-  /* Verify */
-  for(element=new_list_part;element!=NULL;element=element->next)
-  {
-    if(can_be_ext(disk_car,element->part)==0)
-    {
-      nbr_prim++;
-      if((end_log_block!=NULL) && (end_log_block->next==element))
-      {
-	if(log_block_size>biggest_log_block_size)
-	{
-	  biggest_log_block_size=log_block_size;
-	  end_biggest_log_block=end_log_block;
-	}
-	nbr_log_block++;
-	end_log_block=NULL;
-      }
-    }
-    else
-    {
-      log_block_size++;
-      end_log_block=element;
-    }
-  }
-  /* Verification */
-  if((end_log_block!=NULL) && (end_log_block->next==NULL))
-  {
-    if(log_block_size>biggest_log_block_size)
-    {
-      biggest_log_block_size=log_block_size;
-      end_biggest_log_block=end_log_block;
-    }
-    nbr_log_block++;
-  }
-  if(debug>1)
-    ecrit_rapport("\nRes: nbr_prim %u, nbr_log_block %u\n", nbr_prim, nbr_log_block);
-  /* Set primary, extended, logical */
-  if(nbr_prim+nbr_log_block<=4)
-  {
-    int set_prim_bootable_done=0;
-    for(element=end_biggest_log_block;element&&can_be_ext(disk_car,element->part);element=element->prev)
-    {
-/*     ecrit_rapport("%p %p %p %d\n",element,element->prev,element->next,element->part->status); */
-      element->part->status=STATUS_LOG;
-    }
-    for(element=new_list_part;element!=NULL;element=element->next)
-    {
-      if(element->part->status!=STATUS_LOG)
-      {
-	if(set_prim_bootable_done)
-	  element->part->status=STATUS_PRIM;
-	else
-	{
-	  /* The first primary partition is bootable unless it's a swap */
-	  if(element->part->upart_type!=UP_LINSWAP)
-	  {
-	    element->part->status=STATUS_PRIM_BOOT;
-	    set_prim_bootable_done=1;
-	  }
-	}
-      }
-    }
-  }
-  check_list_part(new_list_part);
-  if(test_structure(new_list_part))
-  {
-    for(element=new_list_part;element!=NULL;element=element->next)
-      element->part->status=STATUS_DELETED;
-  }
-  {
-    /* free */
-    t_list_part *new_element;
-    for(element=new_list_part;element!=NULL;element=new_element)
-    {
-      new_element=element->next;
-      FREE(element);
-    }
-  }
-  check_list_part(list_part);
-}
 
 void only_one_bootable( t_list_part *list_part, t_list_part *part_boot)
 {
@@ -194,12 +85,139 @@ void only_one_bootable( t_list_part *list_part, t_list_part *part_boot)
     }
 }
 
+static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
+{
+  int quit=0;
+  int offset=0;
+  int pos_num=0;
+  t_list_part *pos=list_part;
+  if(list_part==NULL)
+    return 1;
+  aff_copy(stdscr);
+  wmove(stdscr,4,0);
+  wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+  wmove(stdscr,6,0);
+  wdoprintf(stdscr,"Check the harddisk size: HD jumpers settings, BIOS detection, OS limitation...");
+  wmove(stdscr,7,0);
+  if(list_part->next==NULL)
+  {
+    wdoprintf(stdscr,"The following partition can't be recovered. The harddisk seems too small!");
+  } else {
+    wdoprintf(stdscr,"The following partitions can't be recovered. The harddisk seems too small!");
+  }
+  mvwaddstr(stdscr,8,0,msg_PART_HEADER);
+  wmove(stdscr,22,0);
+  wattrset(stdscr,A_STANDOUT);
+  wdoprintf(stdscr,"[  Quit  ]");
+  wattroff(stdscr,A_STANDOUT);
+  do
+  {
+    int i;
+    int car;
+    t_list_part *parts;
+    for(i=0,parts=list_part;(parts!=NULL) && (i<offset);parts=parts->next,i++);
+    for(i=offset;(parts!=NULL) &&((i-offset)<INTER_BAD_PART);i++,parts=parts->next)
+    {
+      wmove(stdscr,9+i-offset,0);
+      wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
+      if(parts==pos)
+      {
+	wattrset(stdscr,A_STANDOUT);
+	aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
+	wattroff(stdscr,A_STANDOUT);
+	wmove(stdscr,24,0);
+	wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
+	if(parts->part->info[0]!='\0')
+	{
+	  wdoprintf(stdscr,"%s, ",parts->part->info);
+	}
+	wdoprintf(stdscr,"%lu MB",(long unsigned)(parts->part->part_size/(1024*1024)));
+      } else
+      {
+	aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
+      }
+    }
+    wrefresh(stdscr);
+    car=wgetch(stdscr);
+    switch(car)
+    {
+      case 'q':
+      case '\r':
+      case '\n':
+      case KEY_ENTER:
+#ifdef PADENTER
+      case PADENTER:
+#endif
+      case 'M':
+	quit=1;
+	break;
+      case KEY_UP:
+	if(list_part!=NULL)
+	{
+	  only_one_bootable(list_part,pos);
+	  if(pos->prev!=NULL)
+	  {
+	    pos=pos->prev;
+	    pos_num--;
+	  }
+	  if(pos_num<offset)
+	    offset--;
+	}
+	break;
+      case KEY_DOWN:
+	if(list_part!=NULL)
+	{
+	  only_one_bootable(list_part,pos);
+	  if(pos->next!=NULL)
+	  {
+	    pos=pos->next;
+	    pos_num++;
+	  }
+	  if(pos_num>=offset+INTER_BAD_PART)
+	    offset++;
+	}
+	break;
+      case KEY_PPAGE:
+	if(list_part!=NULL)
+	{
+	  only_one_bootable(list_part,pos);
+	  for(i=0;(i<INTER_BAD_PART) && (pos->prev!=NULL);i++)
+	  {
+	    pos=pos->prev;
+	    pos_num--;
+	    if(pos_num<offset)
+	      offset--;
+	  }
+	}
+	break;
+      case KEY_NPAGE:
+	if(list_part!=NULL)
+	{
+	  only_one_bootable(list_part,pos);
+	  for(i=0;(i<INTER_BAD_PART) && (pos->next!=NULL);i++)
+	  {
+	    pos=pos->next;
+	    pos_num++;
+	    if(pos_num>=offset+INTER_BAD_PART)
+	      offset++;
+	  }
+	}
+	break;
+      default:
+/*	ecrit_rapport("ask_structure car=%x\n",car); */
+	break;
+    }
+  } while(quit==0);
+  return 0;
+}
+
 t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int debug, const int dump_ind, const int fast_mode,const int interface)
 {
   int ind_stop=0;
   t_CHS start;
   t_list_part *list_part=NULL;
-  t_diskext *partition=partition_new();
+  t_list_part *list_part_bad=NULL;
+  t_partition *partition=partition_new();
   if(interface!=0)
   {
     wmove(stdscr,22,0);
@@ -209,7 +227,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
   }
   aff_buffer(BUFFER_RESET,"Q");
   ecrit_rapport("\nsearch_part()\n");
-  ecrit_rapport("%s",disk_car->description(disk_car));
+  ecrit_rapport("%s\n",disk_car->description(disk_car));
   /* ncurses: getch will be a non-blocking call */
   for(start.cylinder=0,start.head=1;(start.cylinder<=disk_car->CHS.cylinder)&&(ind_stop==0);start.cylinder++,start.head=0)
   {
@@ -244,7 +262,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
       {
 	int res=0;
 	partition_reset(partition);	/* must not reset lba */
-	partition->lba=CHS2LBA(disk_car,&start);
+	partition->part_offset=CHS2offset(disk_car,&start);
 	if((start.sector==1) && ((start.head<=2)||(fast_mode==2)))
 	{
 	  part_offset++;
@@ -258,12 +276,15 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	      res=search_type_64(disk_car,partition,debug,dump_ind);
 	      break;
 	    case 3:
-	      res=search_type_2(disk_car,partition,debug,dump_ind);
+	      res=search_type_16(disk_car,partition,debug,dump_ind);
 	      break;
 	    case 4:
-	      res=search_type_1(disk_car,partition,debug,dump_ind);
+	      res=search_type_2(disk_car,partition,debug,dump_ind);
 	      break;
 	    case 5:
+	      res=search_type_1(disk_car,partition,debug,dump_ind);
+	      break;
+	    case 6:
 	      res=search_type_0(disk_car,partition,debug,dump_ind);
 	      sector_inc=1;
 	      break;
@@ -289,13 +310,13 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	{
 	  if(interface!=0)
 	  {
-	    wdoprintf(stdscr,msg_READ_ERROR_AT, start.cylinder,start.head,start.sector,partition->lba);
+	    wdoprintf(stdscr,msg_READ_ERROR_AT, start.cylinder,start.head,start.sector,(unsigned long)(partition->part_offset/disk_car->sector_size));
 	  }
 	}
 	if(res<=0 && paranoid!=0)
 	{
 	  int s_log_block_size;
-	  dword ref_lba=partition->lba;
+	  uint64_t ref_offset=partition->part_offset;
 	  /* try backup superblock */
 	  /* It must be in paranoid mode because it can hide otherwise other partition type */
 	  /* Block size: 1024, 2048 or 4096 bytes (8192 bytes on Alpha systems) */
@@ -305,16 +326,13 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	  {
 	    /* sparse superblock feature: The groups chosen are 0, 1 and powers of 3, 5 and 7. */
 	    /* Checking group 3 */
-	    const dword hd_offset=3*(EXT2_MIN_BLOCK<<s_log_block_size)*8*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)+(s_log_block_size==0?2:0);
-	    if(ref_lba>(3*(EXT2_MIN_BLOCK<<s_log_block_size)*8*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)) &&
-	     LBA2sector(disk_car,ref_lba-hd_offset)==1 &&  
-	     (LBA2head(disk_car,ref_lba-hd_offset)<=2 || fast_mode==2))
+	    const uint64_t hd_offset=3*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)*8*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)+(s_log_block_size==0?2*DEFAULT_SECTOR_SIZE:0);
+	    if(ref_offset>=hd_offset &&
+	     offset2sector(disk_car,ref_offset-hd_offset)==1 &&  
+	     (offset2head(disk_car,ref_offset-hd_offset)<=2 || fast_mode==2))
 	    {
-	      unsigned char buffer[2*SECTOR_SIZE];
-/*      unsigned int block_size=(EXT2_MIN_BLOCK_SIZE<<s_log_block_size); */
-/*      ecrit_rapport("CHS %u/%u/%u %lu %u res=%d\n",start.cylinder,start.head,start.sector,partition->lba,part_offset,res); */
-/*      ecrit_rapport("Block size=%u Group %u Backup superblock at %u ?\n",EXT2_MIN_BLOCK_SIZE<<s_log_block_size,group,group*s_blocks_per_group); */
-	      if(disk_car->read(disk_car,2, &buffer, ref_lba)==0)
+	      unsigned char buffer[2*DEFAULT_SECTOR_SIZE];
+	      if(disk_car->read(disk_car,sizeof(buffer), &buffer, ref_offset)==0)
 	      {
 		const struct ext2_super_block *sb=(const struct ext2_super_block*)&buffer;
 		if(le16(sb->s_block_group_nr)>0)
@@ -334,8 +352,8 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	{
 	  /* Because this superblock is at the end of the partition, you should be
 	   * in slow mode to find it... */
-	  unsigned char buffer[8*SECTOR_SIZE];
-	  if(disk_car->read(disk_car,8, &buffer, partition->lba)!=0)
+	  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+	  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
 	  {
 	    res=-1;
 	  }
@@ -349,25 +367,28 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	}
 	if(res>0)
 	{
-	  if(partition->part_type!=0 && is_extended(partition->part_type)==0 && partition->part_size>1 && partition->lba>0)
+	  aff_part_rapport(disk_car,partition);
+	  if(disk_car->arch->is_part_type_ok(partition->part_type)!=0 && partition->part_size>1 && partition->part_offset>0)
 	  {
-	    dword pos_fin=partition->lba+partition->part_size-1;
-	    aff_part_rapport(disk_car,partition);
-	    if(pos_fin<=disk_car->size)
+	    uint64_t pos_fin=partition->part_offset+partition->part_size-1;
+	    if(pos_fin<=disk_car->disk_size)
 	    {
-	      t_diskext *new_partition=partition_new();
-	      dup_t_diskext(new_partition,partition);
-	      new_partition->status=STATUS_DELETED;
-	      /*	    check_part(disk_car,debug,new_partition); */
-	      aff_part_buffer(AFF_PART_NL,disk_car,new_partition);
-	      list_part=insert_new_partition(list_part,new_partition);
+	      {
+		t_partition *new_partition=partition_new();
+		dup_t_partition(new_partition,partition);
+		new_partition->status=STATUS_DELETED;
+		aff_part_buffer(AFF_PART_ORDER,disk_car,new_partition);
+		/* aff_part_buffer must be before insert_new_partition because
+		   insert_new_partition can free the memory */
+		list_part=insert_new_partition(list_part,new_partition);
+	      }
 	      if(interface)
 	      {
 		aff_buffer(BUFFER_SHOW,"Q");
 	      }
-	      if((paranoid==0) && (partition->lba+partition->part_size-1> CHS2LBA(disk_car,&start)))
+	      if((paranoid==0) && (partition->part_offset+partition->part_size-disk_car->sector_size> CHS2offset(disk_car,&start)))
 	      {
-		LBA2CHS(disk_car,partition->lba+partition->part_size-1,&start);
+		offset2CHS(disk_car,partition->part_offset+partition->part_size-1,&start);
 		part_offset=0;
 		sector_inc=1;
 	      }
@@ -375,13 +396,30 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	      if(debug>0)
 	      {
 		aff_part_rapport(disk_car,partition);
-		if(pos_fin>disk_car->size)
+	      }
+	      if(pos_fin>disk_car->disk_size)
+	      {
+		t_partition *new_partition=partition_new();
+		dup_t_partition(new_partition,partition);
+		new_partition->status=STATUS_DELETED;
+		list_part_bad=insert_new_partition(list_part_bad,new_partition);
+		if(debug>0)
 		{
 		  ecrit_rapport("This partition ends after the disk limits. ");
 		}
-		ecrit_rapport("(start=%lu, size=%lu, end=%lu, disk end=%lu)\n",
-		    partition->lba,partition->part_size, pos_fin,disk_car->size);
 	      }
+	      if(debug>0)
+	      {
+		ecrit_rapport("(start=%lu, size=%lu, end=%lu, disk end=%lu)\n",
+		    (unsigned long)(partition->part_offset/disk_car->sector_size),(unsigned long)(partition->part_size/disk_car->sector_size), (unsigned long)(pos_fin/disk_car->sector_size),(unsigned long)(disk_car->disk_size/disk_car->sector_size));
+	      }
+	    }
+	  }
+	  else
+	  {
+	    if(debug>0)
+	    {
+	      ecrit_rapport("Partition not added.\n");
 	    }
 	  }
 	}
@@ -389,37 +427,18 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
     }
   }
   FREE(partition);
-  return list_part;
-}
-
-static void init_part_order(t_list_part *list_part)
-{
-  int nbr_log=0;
-  int nbr_prim=0;
-  t_list_part *element;
-  for(element=list_part;element!=NULL;element=element->next)
+  if(interface!=0 && list_part_bad!=NULL)
   {
-    switch(element->part->status)
-    {
-      case STATUS_PRIM:
-      case STATUS_PRIM_BOOT:
-      case STATUS_EXT:
-	element->part->order=++nbr_prim;
-	break;
-      case STATUS_LOG:
-	element->part->order=(++nbr_log)+4;
-	break;
-      default:
-	ecrit_rapport("init_part_order: severe error\n");
-	break;
-    }
+    interface_part_bad(disk_car,list_part_bad);
   }
+  delete_list_part(list_part_bad);
+  return list_part;
 }
 
 static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part)
 {
-  t_diskext *table[4];
-  t_diskext *table2[4];
+  t_partition *table[4];
+  t_partition *table2[4];
   int nbr_prim=0;
   int i,pos=0;
   int res;
@@ -431,7 +450,7 @@ static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part)
   wmove(stdscr,4,0);
   wdoprintf(stdscr,"%s",disk_car->description(disk_car));
   mvwaddstr(stdscr,5,0,msg_MBR_ORDER);
-  mvwaddstr(stdscr,6,0,msg_PART_HEADER2);
+  mvwaddstr(stdscr,6,0,msg_PART_HEADER_LONG);
   for(element=list_part;element!=NULL;element=element->next)
   {
     if((element->part->order>0) && (element->part->order<5))
@@ -565,20 +584,40 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
   t_list_part *fin=NULL;
   int nbr_entries=0;
   t_CHS start,end;
-  t_diskext *new_partition;
-  for(element=list_part;element!=NULL;element=element->next)
+  t_partition *new_partition;
+  unsigned int order=0;
+  for(element=list_part;element!=NULL;)
   {
-    if(element->part->status==STATUS_LOG)
+    if(element->part->status==STATUS_EXT)
     {
-      if(deb==NULL)
-      {
-	deb=element;
-	nbr_entries++;
-      }
-      fin=element;
+      /* remove already existing extended partition */
+      t_list_part *next=element->next;
+      if(element->prev!=NULL)
+	element->prev->next=element->next;
+      if(element->next!=NULL)
+	element->next->prev=element->prev;
+      order=element->part->order;
+      if(element==list_part)
+	list_part=next;
+      FREE(element->part);
+      FREE(element);
+      element=next;
     }
     else
-      nbr_entries++;
+    {
+      if(element->part->status==STATUS_LOG)
+      {
+	if(deb==NULL)
+	{
+	  deb=element;
+	  nbr_entries++;
+	}
+	fin=element;
+      }
+      else
+	nbr_entries++;
+      element=element->next;
+    }
   }
   if(deb==NULL)
     return list_part;
@@ -590,7 +629,7 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
     }
     if(deb->prev==NULL)
     {
-      LBA2CHS(disk_car,deb->part->lba-1,&start);
+      offset2CHS(disk_car,deb->part->part_offset-1,&start);
       if((align>0) && (start.cylinder>0||start.head>1))
       {
 	start.cylinder=0;
@@ -600,12 +639,12 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
     }
     else
     {
-      start.cylinder=LBA2cylinder(disk_car,deb->prev->part->lba+deb->prev->part->part_size-1)+1;
+      start.cylinder=offset2cylinder(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1)+1;
       start.head=0;
       start.sector=1;
-      if(CHS2LBA(disk_car,&start)>=deb->part->lba)
+      if(CHS2offset(disk_car,&start)>=deb->part->part_offset)
       {
-	LBA2CHS(disk_car,deb->prev->part->lba+deb->prev->part->part_size-1+1,&start);
+	offset2CHS(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1+1,&start);
 	start.sector=1;
 	start.head++;
 	if(start.head>=disk_car->CHS.head)
@@ -613,9 +652,9 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
 	  start.cylinder++;
 	  start.head=0;
 	}
-	if(CHS2LBA(disk_car,&start)>=deb->part->lba)
+	if(CHS2offset(disk_car,&start)>=deb->part->part_offset)
 	{
-	  LBA2CHS(disk_car,deb->prev->part->lba+deb->prev->part->part_size-1+1,&start);
+	  offset2CHS(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1+1,&start);
 	}
       }
     }
@@ -627,12 +666,12 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
     }
     else
     {
-      end.cylinder=LBA2cylinder(disk_car,fin->next->part->lba)-1; /* 8 october 2002 */
+      end.cylinder=offset2cylinder(disk_car,fin->next->part->part_offset)-1; /* 8 october 2002 */
       end.head=disk_car->CHS.head;
       end.sector=disk_car->CHS.sector;
-      if(CHS2LBA(disk_car,&end)<=fin->part->lba+fin->part->part_size-1)
+      if(CHS2offset(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
       {
-	LBA2CHS(disk_car,fin->next->part->lba-1,&end);
+	offset2CHS(disk_car,fin->next->part->part_offset-1,&end);
 	end.sector=disk_car->CHS.sector;
 	if(end.head>0)
 	  end.head--;
@@ -641,9 +680,9 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
 	  end.cylinder--;
 	  end.head=disk_car->CHS.head;
 	}
-	if(CHS2LBA(disk_car,&end)<=fin->part->lba+fin->part->part_size-1)
+	if(CHS2offset(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
 	{
-	  LBA2CHS(disk_car,fin->next->part->lba-1,&end);
+	  offset2CHS(disk_car,fin->next->part->part_offset-1,&end);
 	}
       }
     }
@@ -654,130 +693,39 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
     {
       ecrit_rapport("add_ext_part: min\n");
     }
-    LBA2CHS(disk_car,deb->part->lba-1,&start);
+    offset2CHS(disk_car,deb->part->part_offset-1,&start);
     start.sector=1;
-    if(start.head>0)
-      start.head--;
-    else
+    if(deb->prev && CHS2offset(disk_car,&start)<=deb->prev->part->part_offset+deb->prev->part->part_size-1)
     {
-      start.head=disk_car->CHS.head;
-      start.cylinder--;
-    }
-    if(deb->prev && CHS2LBA(disk_car,&start)<=deb->prev->part->lba+deb->prev->part->part_size-1)
-    {
-      LBA2CHS(disk_car,deb->part->lba-1,&start);
+      offset2CHS(disk_car,deb->part->part_offset-1,&start);
       start.sector=1;
-      if(CHS2LBA(disk_car,&start)<=deb->prev->part->lba+deb->prev->part->part_size-1)
+      if(CHS2offset(disk_car,&start)<=deb->prev->part->part_offset+deb->prev->part->part_size-1)
       {
-	LBA2CHS(disk_car,deb->part->lba-1,&start);
+	offset2CHS(disk_car,deb->part->part_offset-1,&start);
       }
     }
-    LBA2CHS(disk_car,fin->part->lba+fin->part->part_size-1,&end);
+    offset2CHS(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
     end.head=disk_car->CHS.head;
     end.sector=disk_car->CHS.sector;
-    if(fin->next && CHS2LBA(disk_car,&end)>=fin->next->part->lba)
+    if(fin->next && CHS2offset(disk_car,&end)>=fin->next->part->part_offset)
     {
-      LBA2CHS(disk_car,fin->part->lba+fin->part->part_size-1,&end);
+      offset2CHS(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
       end.sector=disk_car->CHS.sector;
     }
-    if(fin->next && CHS2LBA(disk_car,&end)>=fin->next->part->lba)
+    if(fin->next && CHS2offset(disk_car,&end)>=fin->next->part->part_offset)
     {
-      LBA2CHS(disk_car,fin->part->lba+fin->part->part_size-1,&end);
+      offset2CHS(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
     }
   }
   new_partition=partition_new();
+  new_partition->order=order;
   new_partition->part_type=(end.cylinder>1023?P_EXTENDX:P_EXTENDED);
   new_partition->status=STATUS_EXT;
-  new_partition->lba=CHS2LBA(disk_car,&start);
-  new_partition->part_size=CHS2LBA(disk_car,&end)-new_partition->lba+1;
+  new_partition->part_offset=CHS2offset(disk_car,&start);
+  new_partition->part_size=(uint64_t)CHS2offset(disk_car,&end)-new_partition->part_offset+disk_car->sector_size;
   return insert_new_partition(list_part, new_partition);
 }
 
-static int write_mbr(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug)
-{
-  const t_list_part *element;
-  unsigned char buffer[SECTOR_SIZE];
-  unsigned char buffer_org[SECTOR_SIZE];
-  if(debug>0)
-  {
-    ecrit_rapport("\nwrite_mbr: starting...\n");
-  }
-  if(read_MBR(disk_car,&buffer_org))
-  {
-    memset((void *)&buffer_org,0,SECTOR_SIZE);
-  }
-  memset((void *)&buffer,0,SECTOR_SIZE);
-  if((buffer_org[0x1FE]==0x55) && (buffer_org[0x1FF]==0xAA))
-  {
-    memcpy(buffer,buffer_org,TAB_PART);
-    buffer[0x1FE]=0x55;
-    buffer[0x1FF]=0xAA;
-  } else {
-    write_MBR_code_aux(buffer);
-  }
-  for(element=list_part;element!=NULL;element=element->next)
-  {
-    switch(element->part->status)
-    {
-      case STATUS_PRIM:
-      case STATUS_PRIM_BOOT:
-      case STATUS_EXT:
-	if((element->part->order>=1) && (element->part->order<=4))
-	{
-	  partition2entry(disk_car,0,element->part, 
-	      pt_offset(buffer,element->part->order-1));
-	}
-	break;
-      case STATUS_LOG:
-	break;
-      default:
-	ecrit_rapport("write_mbr: severe error\n");
-	break;
-    }
-  }
-  if(debug>1)
-  {
-    int i;
-    for(i=0;i<4;i++)
-    {
-      const struct partition_dos *p=pt_offset(buffer,i);
-      aff_entry_rapport(p);
-    }
-    diff(buffer,buffer_org);
-  }
-  if(ro==0)
-    return write_MBR(disk_car,&buffer);
-  return 0;
-}
-
-static int diff(const unsigned char buffer[SECTOR_SIZE], const unsigned char buffer_org[SECTOR_SIZE])
-{
-  int j;
-  if(memcmp(buffer,buffer_org,SECTOR_SIZE))
-  {
-    ecrit_rapport("\nSectors are different.\n");
-    ecrit_rapport("buffer_org\n");
-    for(j=0;j<4;j++)
-    {
-      const struct partition_dos *p=pt_offset_const(buffer_org,j);
-      if(p->sys_ind!=0)
-	aff_entry_rapport(p);
-    }
-    ecrit_rapport("buffer\n");
-    for(j=0;j<4;j++)
-    {
-      const struct partition_dos *p=pt_offset_const(buffer,j);
-      if(p->sys_ind!=0)
-	aff_entry_rapport(p);
-    }
-    for(j=0;j<SECTOR_SIZE;j++)
-      if(buffer_org[j]!=buffer[j])
-	ecrit_rapport("%02X %02X %02X\n", j, buffer_org[j], buffer[j]);
-    ecrit_rapport("\n");
-  }
-  return 0;
-}
-
 static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert)
 {
   const t_list_part *element;
@@ -807,144 +755,7 @@ static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, cons
   return 0;
 }
 
-static int write_all_log(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug, const int align)
-{
-  const t_list_part *element;
-  const t_list_part *pos_ext=NULL;
-  dword current_pos;
-  t_diskext *bloc_nextext;
-  if(debug)
-    ecrit_rapport("write_all_log: starting...\n");
-  for(element=list_part;element!=NULL;element=element->next)
-  {
-    if(element->part->status==STATUS_EXT)
-    {
-      if(is_extended(element->part->part_type))
-      {
-	if(pos_ext!=NULL)
-	  ecrit_rapport("write_all_log: pos_ext already defined\n");
-	pos_ext=element;
-      }
-      else
-      {
-	ecrit_rapport("write_all_log: STATUS_EXT with bad part_type\n");
-      }
-    }
-  }
-  if(pos_ext==NULL)
-  {
-    wdoprintf(stdscr,msg_NO_EXT_PART);
-    return 1;
-  }
-  current_pos=pos_ext->part->lba;
-  bloc_nextext=(t_diskext *)MALLOC(sizeof(*bloc_nextext));
-  bloc_nextext->part_type=P_EXTENDED;       /* Never P_EXTENDX */
-  if(pos_ext->next==NULL || (pos_ext->next->part->status!=STATUS_LOG))
-  {
-    unsigned char buffer[SECTOR_SIZE];
-    unsigned char buffer_org[SECTOR_SIZE];
-    if(debug>0)
-    {
-      ecrit_rapport("write_all_log: CHS: %u/%u/%u,lba=%lu\n", LBA2cylinder(disk_car,current_pos), LBA2head(disk_car,current_pos), LBA2sector(disk_car,current_pos),current_pos);
-    }
-    if(disk_car->read(disk_car,1, &buffer_org, current_pos))
-    {
-      memset(buffer_org,0,SECTOR_SIZE);
-    }
-    memset(buffer,0,SECTOR_SIZE);
-    memcpy(buffer,buffer_org,TAB_PART);
-    buffer[0x1FE]=0x55;
-    buffer[0x1FF]=0xAA;
-    if(ro)
-    {
-      if(debug>1)
-	diff(buffer,buffer_org);
-    }
-    else
-    {
-      if(disk_car->write(disk_car,1, &buffer, current_pos))
-      {
-	ecrit_rapport(msg_PART_WR_ERR);
-	display_message(msg_PART_WR_ERR);
-      }
-    }
-  }
-  else
-  {
-    for(element=pos_ext->next;(element!=NULL) && (element->part->status==STATUS_LOG);element=element->next)
-    {
-      int j;
-      unsigned char buffer[SECTOR_SIZE];
-      unsigned char buffer_org[SECTOR_SIZE];
-      if(debug>0)
-      {
-	ecrit_rapport("write_all_log: CHS: %u/%u/%u,lba=%lu\n", LBA2cylinder(disk_car,current_pos), LBA2head(disk_car,current_pos), LBA2sector(disk_car,current_pos),current_pos);
-      }
-      if(disk_car->read(disk_car,1, &buffer_org, current_pos))
-      {
-	memset(buffer_org,0,SECTOR_SIZE);
-      }
-      memset(buffer,0,SECTOR_SIZE);
-      memcpy(buffer,buffer_org,TAB_PART);
-      buffer[0x1FE]=0x55;
-      buffer[0x1FF]=0xAA;
-      partition2entry(disk_car,current_pos,element->part, pt_offset(buffer,0));
-      if(element->next!=NULL && (element->next->part->status==STATUS_LOG))
-      { /* Construit le pointeur vers la prochaine partition logique */
-	t_CHS nextext_start;
-	bloc_nextext->lba=element->next->part->lba-1;
-	LBA2CHS(disk_car,bloc_nextext->lba,&nextext_start);
-	nextext_start.sector=1;
-	if(nextext_start.head>0)
-	  nextext_start.head--;
-	else
-	{
-	  nextext_start.head=disk_car->CHS.head;
-	  nextext_start.cylinder--;
-	}
-	if(CHS2LBA(disk_car,&nextext_start)<=element->part->lba+element->part->part_size+1)
-	{
-	  LBA2CHS(disk_car,bloc_nextext->lba,&nextext_start);
-	  nextext_start.sector=1;
-	  if(CHS2LBA(disk_car,&nextext_start)<=element->part->lba+element->part->part_size+1)
-	  {
-	    LBA2CHS(disk_car,bloc_nextext->lba,&nextext_start);
-	  }
-	}
-	bloc_nextext->lba=CHS2LBA(disk_car,&nextext_start);
-	/*      ecrit_rapport("table[i]->next=%p table[i+1]=%p\n",table[i]->next,table[i+1]); */
-	bloc_nextext->part_size=(element->next->part->lba+element->next->part->part_size-1)-bloc_nextext->lba+1;
-	partition2entry(disk_car,pos_ext->part->lba,bloc_nextext, pt_offset(buffer,1));
-      }
-      if(ro)
-      {
-	if(debug>1)
-	{
-	  for(j=0;j<4;j++)
-	  {
-	    const struct partition_dos *p=pt_offset(buffer,j);
-	    if(p->sys_ind!=0)
-	      aff_entry_rapport(p);
-	  }
-	  diff(buffer,buffer_org);
-	}
-      }
-      else
-      {
-	if(disk_car->write(disk_car,1, &buffer, current_pos))
-	{
-	  ecrit_rapport(msg_PART_WR_ERR);
-	  display_message(msg_PART_WR_ERR);
-	}
-      }
-      current_pos=bloc_nextext->lba;
-    }
-  }
-  FREE(bloc_nextext);
-  return 0;
-}
-
-int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const int dump_ind, int fast_mode, const int align, const int ask_part_order, const int max_ext, const unsigned int expert)
+int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const int dump_ind, int fast_mode, const int align, const int ask_part_order, const unsigned int expert)
 {
   int res_interface_write;
   do
@@ -965,19 +776,18 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
     /* Little trick to detect Linux software Raid 1 quickly */
     for(element=list_part;element!=NULL;element=element->next)
     {
-      unsigned char buffer[8*SECTOR_SIZE];
+      unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
       struct mdp_superblock_s *sb=(struct mdp_superblock_s*)&buffer;
-      dword offset=MD_NEW_SIZE_SECTORS(element->part->part_size);
-      
-      if(disk_car->read(disk_car,8, &buffer, element->part->lba+offset)==0)
+      uint64_t offset=MD_NEW_SIZE_SECTORS(element->part->part_size/disk_car->sector_size)*disk_car->sector_size;
+      if(disk_car->read(disk_car,sizeof(buffer), &buffer, element->part->part_offset+offset)==0)
       {
-	t_diskext *new_partition=partition_new();
-	dup_t_diskext(new_partition,element->part);
+	t_partition *new_partition=partition_new();
+	dup_t_partition(new_partition,element->part);
 	if(test_MD(disk_car,sb,new_partition,debug,dump_ind)==0)
 	{
 	  set_MD_info(disk_car,sb,new_partition,debug,dump_ind);
 	  new_partition->part_type=P_RAID;
-	  new_partition->part_size=(sb->size<<1)+MD_RESERVED_SECTORS;
+	  new_partition->part_size=(uint64_t)(sb->size<<1)*disk_car->sector_size+MD_RESERVED_BYTES;
 	  list_part=insert_new_partition(list_part,new_partition);
 	}
 	else
@@ -985,11 +795,11 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
       }
     }
     align_structure(disk_car,list_part,align);
-    init_structure(disk_car,list_part,debug);
+    disk_car->arch->init_structure(disk_car,list_part,debug);
 #ifdef DEBUG
     check_list_part(list_part);
 #endif
-    if(debug)
+    if(debug>0)
     {
       /* Ajoute dans le fichier de log les partitions trouvees */
       ecrit_rapport("\nResults\n");
@@ -998,8 +808,11 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
     }
     wdoprintf(stdscr,msg_GOD_MODE);
     list_part=ask_structure(disk_car,list_part,debug);
-    if(test_structure(list_part)==0)
+    if(disk_car->arch->test_structure(list_part)==0)
     {
+      int do_again=0;
+      int max_ext=0;
+      int can_ask_minmax_ext=0;
       list_part=reduce_structure(list_part);
 #ifdef DEBUG
       check_list_part(list_part);
@@ -1010,46 +823,76 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
       check_list_part(list_part);
 #endif
       /* Creer la partition etendue globale, cherche  aligner la partition */
-      list_part=add_ext_part(disk_car,list_part,max_ext,2,debug);
-      /*
-      ecrit_rapport("\nResults with extended\n");
-      for(element=list_part;element;element=element->next)
-	aff_part_rapport(disk_car,element->part);
-	*/
-      init_part_order(list_part);
-      if(ask_part_order)
+      {
+	t_list_part *parts;
+	uint64_t partext_offset=0;
+	uint64_t partext_size=0;
+	list_part=add_ext_part(disk_car,list_part,!max_ext,2,debug);
+	for(parts=list_part;parts!=NULL;parts=parts->next)
+	  if(parts->part->status==STATUS_EXT)
+	  {
+	    partext_offset=parts->part->part_offset;
+	    partext_size=parts->part->part_size;
+	  }
+#ifdef DEBUG
+	check_list_part(list_part);
+#endif
+	if(partext_offset>0)
+	{
+	  list_part=add_ext_part(disk_car,list_part,max_ext,2,debug);
+	  for(parts=list_part;parts!=NULL;parts=parts->next)
+	    if(parts->part->status==STATUS_EXT)
+	    {
+	      if(partext_offset!=parts->part->part_offset || partext_size!=parts->part->part_size)
+		can_ask_minmax_ext=1;
+	    }
+	}
+#ifdef DEBUG
+	check_list_part(list_part);
+#endif
+      }
+      list_part=disk_car->arch->init_part_order(disk_car,list_part);
+      if(ask_part_order!=0)
       {
 	/* Demande l'ordre des entrees dans le MBR */
 	ask_mbr_order(disk_car,list_part);
 	/* Demande l'ordre des partitions etendues */
       }
-      res_interface_write=interface_write(disk_car,list_part,can_search_deeper);
-      switch(res_interface_write)
+      do
       {
-	case 1:
-	  if(ask_confirmation("Write partition table, confirm ? (Y/N)"))
-	  {
-	    ecrit_rapport("write!\n");
-	    write_mbr(disk_car,list_part,RW,debug);
-	    write_all_log(disk_car,list_part,RW,debug,align);
-	    use_backup(disk_car,list_part,debug,dump_ind,expert);
-	  }
-	  else
-	    ecrit_rapport("Don't write, no confirmation\n");
-	break;
-	case 0:
-	ecrit_rapport("simulate write!\n");
-	write_mbr(disk_car,list_part,RO,debug);
-	write_all_log(disk_car,list_part,RO,debug,align);
-	break;
-	case 2:
-	paranoid=1;
-	if(fast_mode<2)
-	  fast_mode++;
-	break;
+	do_again=0;
+	res_interface_write=interface_write(disk_car,list_part,can_search_deeper,can_ask_minmax_ext);
+	switch(res_interface_write)
+	{
+	  case 'W':
+	    if(ask_confirmation("Write partition table, confirm ? (Y/N)"))
+	    {
+	      ecrit_rapport("write!\n");
+	      disk_car->arch->write_part(disk_car,list_part,RW,debug,align);
+	      use_backup(disk_car,list_part,debug,dump_ind,expert);
+	    }
+	    else
+	      ecrit_rapport("Don't write, no confirmation\n");
+	    break;
+	  case 0:
+	    ecrit_rapport("simulate write!\n");
+	    disk_car->arch->write_part(disk_car,list_part,RO,debug,align);
+	    break;
+	  case 'S':
+	    paranoid=1;
+	    if(fast_mode<2)
+	      fast_mode++;
+	    break;
+	  case 'E':
+	    max_ext=!max_ext;
+	    list_part=add_ext_part(disk_car,list_part,max_ext,2,debug);
+	    do_again=1;
+	    break;
+	}
       }
+      while(do_again==1);
     }
     delete_list_part(list_part);
-  } while(res_interface_write==2);
+  } while(res_interface_write=='S');
   return 0;
 }
