@@ -20,6 +20,6 @@
 
  */
 #define L2BPERDMAP 13      /* l2 num of blks per dmap */
-int check_JFS(t_param_disk *disk_car,t_diskext *partition,const int debug);
-int recover_JFS(t_param_disk *disk_car, const struct jfs_superblock *sb,t_diskext *partition,const int debug, const int dump_ind);
-int set_JFS_info(t_param_disk *disk_car, const struct jfs_superblock *sb,t_diskext *partition,const int debug, const int dump_ind);
+int check_JFS(t_param_disk *disk_car,t_partition *partition,const int debug);
+int recover_JFS(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partition *partition,const int debug, const int dump_ind);
+int set_JFS_info(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partition *partition,const int debug, const int dump_ind);
