@@ -20,7 +20,6 @@
 
  */
 
-#define msg_CMDLINE       "TestDisk command line :"
 #define msg_DUMP_HEXA     "Dump Hexa\n"
 #define msg_READ_ERROR_AT "\nRead error at %u/%u/%u (lba=%lu)"
 #define c_YES           'Y'
@@ -39,22 +38,18 @@
 #define msg_BAD_SCOUNT    "\nBad sector count."
 #define msg_CHKFAT_RERR   "check_FAT: Read error\n"
 #define msg_CHKFAT_BAD_JUMP       "check_FAT: Bad jump in FAT partition\n"
-#define msg_CHKFAT_BYTESECT       "check_FAT: Incorrect number of bytes per sector\n"
 #define msg_CHKFAT_SECT_CLUSTER   "check_FAT: Bad number of sectors per cluster\n"
 #define msg_CHKFAT_SECTTRACK      "check_FAT: Incorrect number of sectors per track\n"
 #define msg_CHKFAT_SIDE   "check_FAT: Incorrect number of sides\n"
 #define msg_CHKFAT_NBRFAT "check_FAT: Bad number of FAT\n"
 #define msg_CHKFAT_ENTRY  "check_FAT: Bad number of entries in root dir\n"
-#define msg_CHKFAT_MEDIA  "check_FAT: Bad descriptor\n"
 #define msg_CHKFAT_SIZE   "check_FAT: Incorrect size of partition\n"
-#define msg_CHKFAT_RESERV "check_FAT: Incorrect number of reserved sectors\n"
 #define msg_CHKFAT_SECTPFAT "check_FAT: Incorrect number of sectors per FAT\n"
 #define msg_CHKFAT_BAD_HIDDEN "check_FAT: Incorrect number of hidden sectors\n"
 #define msg_CHKFAT_BADFAT32VERSION "check_FAT: Bad FAT32 version, should be 0.0\n"
-#define msg_PART_HEADER   "     Partition\t\t\t Start          End       Size\n"
-#define msg_PART_HEADER2   "    Partition\t\t      Start          End       Size\n"
-#define msg_PART_HEADER_WRITE   "     Partition\t\t\t Start          End       Size\n"
-#define msg_GOD_MODE      "\nGod mode\n\n     Partition\t\t\t     Start        End        Size"
+#define msg_PART_HEADER   	"     Partition\t\t     Start        End    Size in sectors\n"
+#define msg_PART_HEADER_LONG   	"     Partition\t\t\tStart        End    Size in sectors\n"
+#define msg_GOD_MODE      "\nGod mode\n\n     Partition\t\t\t     Start     End        Size in sectors"
 #define msg_PART_RD_ERR   "\nPartition : Read error\n"
 #define msg_PART_WR_ERR   "\nPartition : Write error\n"
 #define msg_ONLY_ONE_DOS  "Partition Table contains two or more Primary DOS FAT partitions\n"
@@ -63,39 +58,19 @@
 #define msg_NO_BOOTABLE   "No partition is bootable\n"
 #define msg_ONLY1MUSTBOOT "Only one partition must be bootable\n"
 #define msg_SAME_SPACE    "Space conflict between the following two partitions\n"
-#define msg_LOG_ERR       "\nCan not create testdisk.log\n"
 #define msg_DISK		"Disk"
-#define msg_CHOOSE_PART_TYPE "Use arrow keys to change partition characteristics (see below)"
-#define msg_CHOOSE_PART_TYPE_END "Press the ENTER key to continue"
 #define msg_CHOICE        "\nChoice ? "
 #define msg_MALLOC_ERROR  "\nMemory allocation error"
 #define msg_FREE_ERROR    "\nFree memory allocation error"
 #define msg_WRITE_CLEAN_TABLE     "Clear MBR partition table by writing zero bytes to it? (Y/N) "
 #define msg_WRITE_MBR_CODE    "Write a new copy of MBR code to first sector? (Y/N) "
-#define msg_STRUCT_BAD    "Structure: Bad"
-#define msg_STRUCT_OK             "Structure: Ok "
+#define msg_STRUCT_BAD    	"Structure: Bad."
+#define msg_STRUCT_OK           "Structure: Ok. "
 #define msg_MBR_ORDER             "MBR order - Use 1234 to change the order and ENTER to continue"
 #define msg_MBR_ORDER_GOOD        "Partitions order: Ok "
 #define msg_MBR_ORDER_BAD "Partitions order: Bad"
 #define msg_NO_EXT_PART   "\nNo extended partition"
 #define msg_ROOT_CLUSTER_RERR	"\nroot_cluster: read error"
-#define msg_Usage	"\nUsage: TestDisk [/log] [/debug] [/dump] [file or device]"\
-		"\n       TestDisk /list [/log] [file or device]" \
-	    "\n" \
-	    "\n/log          : create a testdisk.log file" \
-	    "\n/debug        : add debug information" \
-	    "\n/dump         : dump raw sectors" \
-	    "\n/list         : display current partitions" \
-	    "\n" \
-	    "\nTestDisk checks and recovers lost partitions" \
-	    "\nIt works with :" \
-	    "\n- FAT12, FAT16 <32M, FAT16 >32M, FAT32" \
-	    "\n- NTFS" \
-	    "\n- LINUX ext2fs/ext3fs, LINUX swap" \
-	    "\n- OS/2 MultiBoot (also used by Partition Magic)" \
-	    "\n- BFS (BeOS)" \
-	    "\n- UFS (BSD)" \
-	    "\n- ReiserFS (1 & 2)" \
-	    "\n" \
-	    "\nIf you have problems with TestDisk or bug reports, please contacte me.\n"
 #define msg_PART_TYPE "*=Primary bootable  P=Primary  L=Logical  E=Extended  D=Deleted"
+const char *partition_type(const unsigned char type, const struct systypes *parttype_name_table);
+
