@@ -24,45 +24,50 @@
 #endif
  
 #include <stdio.h>
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "lvm.h"
-#include "intrface.h"
+#include "intrf.h"
 #include "fnctdsk.h"
-int check_LVM(t_param_disk *disk_car,t_diskext *partition,const int debug)
+int check_LVM(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
-  unsigned char buffer[8*SECTOR_SIZE];
-  if(disk_car->read(disk_car,8, &buffer, partition->lba)!=0)
+  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
   { return 1; }
   if(test_LVM(disk_car,(pv_disk_t *)&buffer,partition,debug,0)!=0)
     return 1;
   return 0;
 }
 
-int recover_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_diskext *partition,const int debug, const int dump_ind)
+int recover_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_partition *partition,const int debug, const int dump_ind)
 {
   if(test_LVM(disk_car,pv,partition,debug,dump_ind)!=0)
     return 1;
   partition->part_type=(unsigned char)P_LVM;
-  partition->part_size=(dword)pv->pv_size;
+  partition->part_size=(uint64_t)pv->pv_size*disk_car->sector_size;
   if(debug!=0)
   {
-    ecrit_rapport("part_size %lu\n",partition->part_size);
+    ecrit_rapport("part_size %lu\n",(long unsigned)(partition->part_size/disk_car->sector_size));
   }
   return 0;
 }
 
-int test_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_diskext *partition,const int debug, const int dump_ind)
+int test_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_partition *partition,const int debug, const int dump_ind)
 {
   if ((strncmp((const char *)pv->id,LVM_ID,sizeof(pv->id)) == 0) && ((pv->version == 1) || (pv->version == 2)))
   {
     uint32_t size;
+    if(debug>0 || dump_ind!=0)
+    {
+      ecrit_rapport("\nLVM magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+    }
     if(dump_ind!=0)
     {
-      ecrit_rapport("\nLVM magic value at %u/%u/%u\n", LBA2cylinder(disk_car,partition->lba),LBA2head(disk_car,partition->lba),LBA2sector(disk_car,partition->lba));
       /* There is a little offset ... */
-/*     dump(stdscr,pv,SECTOR_SIZE); */
+      dump(stdscr,pv,DEFAULT_SECTOR_SIZE);
     }
     if (pv->pv_size > LVM_MAX_SIZE)
       return (1);
