@@ -26,13 +26,6 @@
  * structs & defines gathered from LVM 0.7/0.9 lvm.h and liblvm.h
  */
 
-#if !defined(__FreeBSD__) && !defined(__CYGWIN__)
-typedef unsigned char uint8_t;
-typedef unsigned short uint16_t;
-typedef unsigned int uint32_t;
-#endif
-
-
 /*
  * Status flags
  */
@@ -93,9 +86,9 @@ typedef struct {
 } pv_disk_v2_t;
 
 #define pv_disk_t pv_disk_v2_t
-int check_LVM(t_param_disk *disk_car,t_diskext *partition,const int debug);
-int test_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_diskext *partition,const int debug, const int dump_ind);
-int recover_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_diskext *partition,const int debug, const int dump_ind);
+int check_LVM(t_param_disk *disk_car,t_partition *partition,const int debug);
+int test_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_partition *partition,const int debug, const int dump_ind);
+int recover_LVM(t_param_disk *disk_car, const pv_disk_t *pv,t_partition *partition,const int debug, const int dump_ind);
 
 
 #endif /* _GM_HMLVM_H */
