@@ -24,17 +24,19 @@
 #endif
  
 #include <stdio.h>
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "md.h"
-#include "intrface.h"
+#include "intrf.h"
 #include "fnctdsk.h"
-int check_MD(t_param_disk *disk_car,t_diskext *partition,const int debug)
+int check_MD(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
-  unsigned char buffer[8*SECTOR_SIZE];
-  dword offset=MD_NEW_SIZE_SECTORS(partition->part_size);
-  if(disk_car->read(disk_car,8, &buffer, partition->lba+offset)!=0)
+  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+  uint64_t offset=MD_NEW_SIZE_SECTORS(partition->part_size/disk_car->sector_size)*disk_car->sector_size;
+  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+offset)!=0)
   { return 1; }
   if(test_MD(disk_car,(struct mdp_superblock_s*)&buffer,partition,debug,0)!=0)
     return 1;
@@ -42,18 +44,18 @@ int check_MD(t_param_disk *disk_car,t_diskext *partition,const int debug)
   return 0;
 }
 
-int set_MD_info(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_diskext *partition,const int debug, const int dump_ind)
+int set_MD_info(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partition *partition,const int debug, const int dump_ind)
 {
   unsigned int i;
-  sprintf(partition->name,"md%u",sb->md_minor);
-  sprintf(partition->info,"Raid %u: devices",sb->level);
+  sprintf(partition->name,"md%u",(unsigned int)sb->md_minor);
+  sprintf(partition->info,"Raid %u: devices",(unsigned int)sb->level);
   for(i=0;i<MD_SB_DISKS;i++)
   {
     if(sb->disks[i].major!=0 && sb->disks[i].minor!=0)
     {
       if(strlen(partition->info)<sizeof(partition->info)-26)
       { 
-	sprintf(&partition->info[strlen(partition->info)]," (%u,%u)",sb->disks[i].major,sb->disks[i].minor);
+	sprintf(&partition->info[strlen(partition->info)]," (%u,%u)",(unsigned int)sb->disks[i].major,(unsigned int)sb->disks[i].minor);
 	if(sb->disks[i].major==sb->this_disk.major && sb->disks[i].minor==sb->this_disk.minor)
 	  sprintf(&partition->info[strlen(partition->info)],"*");
       }
@@ -62,26 +64,26 @@ int set_MD_info(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_disk
   return 0;
 }
 
-int recover_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_diskext *partition,const int debug, const int dump_ind)
+int recover_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partition *partition,const int debug, const int dump_ind)
 {
   if(test_MD(disk_car,sb,partition,debug,dump_ind)!=0)
     return 1;
   set_MD_info(disk_car,sb,partition,debug,dump_ind);
   partition->part_type=(unsigned char)P_RAID;
-  partition->lba=partition->lba-(sb->size<<1);
-  partition->part_size=(sb->size<<1)+MD_RESERVED_SECTORS;
+  partition->part_offset=partition->part_offset-(sb->size<<1);
+  partition->part_size=(uint64_t)(sb->size<<1)*disk_car->sector_size+MD_RESERVED_BYTES;
   return 0;
 }
 
-int test_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_diskext *partition,const int debug, const int dump_ind)
+int test_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partition *partition,const int debug, const int dump_ind)
 {
   if(sb->md_magic==(unsigned int)MD_SB_MAGIC)
   {
     if(dump_ind!=0)
     {
-      ecrit_rapport("\nRaid magic value at %u/%u/%u\n", LBA2cylinder(disk_car,partition->lba),LBA2head(disk_car,partition->lba),LBA2sector(disk_car,partition->lba));
+      ecrit_rapport("\nRaid magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
       /* There is a little offset ... */
-      dump(stdscr,sb,SECTOR_SIZE);
+      dump(stdscr,sb,DEFAULT_SECTOR_SIZE);
     }
     partition->upart_type=UP_RAID;
     return 0;
