@@ -92,12 +92,12 @@
 #define MD_DISK_REMOVED		3 /* disk is in sync with the raid set */
 
 typedef struct mdp_device_descriptor_s {
-	__u32 number;		/* 0 Device number in the entire set	      */
-	__u32 major;		/* 1 Device major number		      */
-	__u32 minor;		/* 2 Device minor number		      */
-	__u32 raid_disk;	/* 3 The role of the device in the raid set   */
-	__u32 state;		/* 4 Operational state			      */
-	__u32 reserved[MD_SB_DESCRIPTOR_WORDS - 5];
+	uint32_t number;		/* 0 Device number in the entire set	      */
+	uint32_t major;		/* 1 Device major number		      */
+	uint32_t minor;		/* 2 Device minor number		      */
+	uint32_t raid_disk;	/* 3 The role of the device in the raid set   */
+	uint32_t state;		/* 4 Operational state			      */
+	uint32_t reserved[MD_SB_DESCRIPTOR_WORDS - 5];
 } mdp_disk_t;
 
 #define MD_SB_MAGIC		0xa92b4efc
@@ -112,51 +112,51 @@ typedef struct mdp_superblock_s {
 	/*
 	 * Constant generic information
 	 */
-	__u32 md_magic;		/*  0 MD identifier 			      */
-	__u32 major_version;	/*  1 major version to which the set conforms */
-	__u32 minor_version;	/*  2 minor version ...			      */
-	__u32 patch_version;	/*  3 patchlevel version ...		      */
-	__u32 gvalid_words;	/*  4 Number of used words in this section    */
-	__u32 set_uuid0;	/*  5 Raid set identifier		      */
-	__u32 ctime;		/*  6 Creation time			      */
-	__u32 level;		/*  7 Raid personality			      */
-	__u32 size;		/*  8 Apparent size of each individual disk   */
-	__u32 nr_disks;		/*  9 total disks in the raid set	      */
-	__u32 raid_disks;	/* 10 disks in a fully functional raid set    */
-	__u32 md_minor;		/* 11 preferred MD minor device number	      */
-	__u32 not_persistent;	/* 12 does it have a persistent superblock    */
-	__u32 set_uuid1;	/* 13 Raid set identifier #2		      */
-	__u32 set_uuid2;	/* 14 Raid set identifier #3		      */
-	__u32 set_uuid3;	/* 15 Raid set identifier #4		      */
-	__u32 gstate_creserved[MD_SB_GENERIC_CONSTANT_WORDS - 16];
+	uint32_t md_magic;		/*  0 MD identifier 			      */
+	uint32_t major_version;	/*  1 major version to which the set conforms */
+	uint32_t minor_version;	/*  2 minor version ...			      */
+	uint32_t patch_version;	/*  3 patchlevel version ...		      */
+	uint32_t gvalid_words;	/*  4 Number of used words in this section    */
+	uint32_t set_uuid0;	/*  5 Raid set identifier		      */
+	uint32_t ctime;		/*  6 Creation time			      */
+	uint32_t level;		/*  7 Raid personality			      */
+	uint32_t size;		/*  8 Apparent size of each individual disk   */
+	uint32_t nr_disks;		/*  9 total disks in the raid set	      */
+	uint32_t raid_disks;	/* 10 disks in a fully functional raid set    */
+	uint32_t md_minor;		/* 11 preferred MD minor device number	      */
+	uint32_t not_persistent;	/* 12 does it have a persistent superblock    */
+	uint32_t set_uuid1;	/* 13 Raid set identifier #2		      */
+	uint32_t set_uuid2;	/* 14 Raid set identifier #3		      */
+	uint32_t set_uuid3;	/* 15 Raid set identifier #4		      */
+	uint32_t gstate_creserved[MD_SB_GENERIC_CONSTANT_WORDS - 16];
 
 	/*
 	 * Generic state information
 	 */
-	__u32 utime;		/*  0 Superblock update time		      */
-	__u32 state;		/*  1 State bits (clean, ...)		      */
-	__u32 active_disks;	/*  2 Number of currently active disks	      */
-	__u32 working_disks;	/*  3 Number of working disks		      */
-	__u32 failed_disks;	/*  4 Number of failed disks		      */
-	__u32 spare_disks;	/*  5 Number of spare disks		      */
-	__u32 sb_csum;		/*  6 checksum of the whole superblock        */
+	uint32_t utime;		/*  0 Superblock update time		      */
+	uint32_t state;		/*  1 State bits (clean, ...)		      */
+	uint32_t active_disks;	/*  2 Number of currently active disks	      */
+	uint32_t working_disks;	/*  3 Number of working disks		      */
+	uint32_t failed_disks;	/*  4 Number of failed disks		      */
+	uint32_t spare_disks;	/*  5 Number of spare disks		      */
+	uint32_t sb_csum;		/*  6 checksum of the whole superblock        */
 #ifdef __BIG_ENDIAN
-	__u32 events_hi;	/*  7 high-order of superblock update count   */
-	__u32 events_lo;	/*  8 low-order of superblock update count    */
+	uint32_t events_hi;	/*  7 high-order of superblock update count   */
+	uint32_t events_lo;	/*  8 low-order of superblock update count    */
 #else
-	__u32 events_lo;	/*  7 low-order of superblock update count    */
-	__u32 events_hi;	/*  8 high-order of superblock update count   */
+	uint32_t events_lo;	/*  7 low-order of superblock update count    */
+	uint32_t events_hi;	/*  8 high-order of superblock update count   */
 #endif
-	__u32 gstate_sreserved[MD_SB_GENERIC_STATE_WORDS - 9];
+	uint32_t gstate_sreserved[MD_SB_GENERIC_STATE_WORDS - 9];
 
 	/*
 	 * Personality information
 	 */
-	__u32 layout;		/*  0 the array's physical layout	      */
-	__u32 chunk_size;	/*  1 chunk size in bytes		      */
-	__u32 root_pv;		/*  2 LV root PV */
-	__u32 root_block;	/*  3 LV root block */
-	__u32 pstate_reserved[MD_SB_PERSONALITY_WORDS - 4];
+	uint32_t layout;		/*  0 the array's physical layout	      */
+	uint32_t chunk_size;	/*  1 chunk size in bytes		      */
+	uint32_t root_pv;		/*  2 LV root PV */
+	uint32_t root_block;	/*  3 LV root block */
+	uint32_t pstate_reserved[MD_SB_PERSONALITY_WORDS - 4];
 
 	/*
 	 * Disks information
@@ -166,7 +166,7 @@ typedef struct mdp_superblock_s {
 	/*
 	 * Reserved
 	 */
-	__u32 reserved[MD_SB_RESERVED_WORDS];
+	uint32_t reserved[MD_SB_RESERVED_WORDS];
 
 	/*
 	 * Active descriptor
@@ -175,15 +175,15 @@ typedef struct mdp_superblock_s {
 
 } mdp_super_t;
 
-static inline __u64 md_event(mdp_super_t *sb) {
-	__u64 ev = sb->events_hi;
+static inline uint64_t md_event(mdp_super_t *sb) {
+	uint64_t ev = sb->events_hi;
 	return (ev<<32)| sb->events_lo;
 }
 
 /* TestDisk */
-int check_MD(t_param_disk *disk_car,t_diskext *partition,const int debug);
-int test_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_diskext *partition,const int debug, const int dump_ind);
-int recover_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_diskext *partition,const int debug, const int dump_ind);
-int set_MD_info(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_diskext *partition,const int debug, const int dump_ind);
+int check_MD(t_param_disk *disk_car,t_partition *partition,const int debug);
+int test_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partition *partition,const int debug, const int dump_ind);
+int recover_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partition *partition,const int debug, const int dump_ind);
+int set_MD_info(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partition *partition,const int debug, const int dump_ind);
 #endif 
 
