@@ -23,16 +23,18 @@
 #include <config.h>
 #endif
  
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "netware.h"
 
-int recover_netware(const struct disk_netware *netware_block,t_diskext *partition)
+int recover_netware(t_param_disk *disk_car, const struct disk_netware *netware_block,t_partition *partition)
 {
   if(memcmp(netware_block->magic,"Nw_PaRtItIoN",12)==0)
   {
-    partition->part_size=(dword)netware_block->nbr_sectors;
+    partition->part_size=(uint64_t)netware_block->nbr_sectors*disk_car->sector_size;
     partition->part_type=(unsigned char)P_NETWARE;
     partition->upart_type=UP_NETWARE;
     return 0;
