@@ -22,11 +22,11 @@
 
 typedef struct disk_netware
 {
-    char unknown;
-	char magic[12];
-	char unknown2[3];
-	char unknown3[3]; /* 0x10 */
-	int32 nbr_sectors;
+  char unknown;
+  char magic[12];
+  char unknown2[3];
+  char unknown3[3]; /* 0x10 */
+  int32_t nbr_sectors;
 } disk_netware;
 
-int recover_netware(const struct disk_netware *,t_diskext *);
+int recover_netware(t_param_disk *disk_car, const struct disk_netware *,t_partition *);
