@@ -21,39 +21,45 @@
  */
 /*define NTFS_DEBUG 1 */
 
-#define NTFS_GETU8(p)      (*(const __u8*)(p))
-#define NTFS_GETU16(p)     (*(const __u16*)(p))
-#define NTFS_GETU32(p)     (*(const __u32*)(p))
-#define NTFS_GETU64(p)     (*(const __u64*)(p))
+#define NTFS_GETU8(p)      (*(const uint8_t*)(p))
+#define NTFS_GETU16(p)     (*(const uint16_t*)(p))
+#define NTFS_GETU32(p)     (*(const uint32_t*)(p))
+#define NTFS_GETU64(p)     (*(const uint64_t*)(p))
  
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
  
 #include <stdio.h>
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include <ctype.h>
 #include "types.h"
 #include "common.h"
-#include "intrface.h"
+#include "intrf.h"
 #include "ntfs.h"
 #include "fnctdsk.h"
 #include "lang.h"
 static int testdisk_ffs(int x);
-static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *partition, const int interface, const unsigned int cluster_size, const __u64 mft_lcn, const __u64 mftmirr_lcn, const __s32 mft_record_size, const __s32 index_block_size);
+static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_partition *partition, const int interface, const unsigned int cluster_size, const uint64_t mft_lcn, const uint64_t mftmirr_lcn, const int32_t mft_record_size, const int32_t index_block_size);
 static int dump_ntfs_info(const struct ntfs_boot_sector *ntfs_header);
 static int dump_2ntfs_info(const struct ntfs_boot_sector *nh1, const struct ntfs_boot_sector *nh2);
 
-static int ntfs_read_MFT(t_param_disk *disk_car, t_diskext *partition, const struct ntfs_boot_sector*ntfs_header,const int my_type,const int debug, const int dump_ind);
-static int ntfs_find_mft(t_param_disk *disk_car, t_diskext *partition, const int debug, const int interface);
+static int ntfs_read_MFT(t_param_disk *disk_car, t_partition *partition, const struct ntfs_boot_sector*ntfs_header,const int my_type,const int debug, const int dump_ind);
 
-static int ntfs_get_attr(const char *mft_record, const int my_type, t_diskext *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find);
-static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_diskext *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find);
+static int ntfs_get_attr(const char *mft_record, const int my_type, t_partition *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find);
+static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_partition *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find);
+unsigned int ntfs_sector_size(const struct ntfs_boot_sector *ntfs_header);
 
-int check_NTFS(t_param_disk *disk_car,t_diskext *partition,const int debug,const int dump_ind)
+unsigned int ntfs_sector_size(const struct ntfs_boot_sector *ntfs_header)
+{ return (ntfs_header->sector_size[1]<<8)+ntfs_header->sector_size[0]; }
+
+int check_NTFS(t_param_disk *disk_car,t_partition *partition,const int debug,const int dump_ind)
 {
-  unsigned char buffer[SECTOR_SIZE];
-  if(disk_car->read(disk_car,1, &buffer, partition->lba)!=0)
+  unsigned char buffer[DEFAULT_SECTOR_SIZE];
+/*  ecrit_rapport("check_NTFS part_offset=%llu\n",(long long unsigned)partition->part_offset); */
+  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
   { return 1; }
   if(test_NTFS(disk_car,(struct ntfs_boot_sector*)&buffer,partition,debug,dump_ind)!=0)
     return 1;
@@ -61,32 +67,39 @@ int check_NTFS(t_param_disk *disk_car,t_diskext *partition,const int debug,const
   return 0;
 }
 
-int recover_NTFS(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_diskext *partition,const int debug, const int dump_ind, const int backup)
+int recover_NTFS(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind, const int backup)
 {
   if(test_NTFS(disk_car,ntfs_header,partition,debug,dump_ind)!=0)
     return 1;
-  if(debug!=0)
+  if(debug>0)
   {
     dump_ntfs_rapport(ntfs_header);
   }
-  partition->part_size=le64(ntfs_header->sectors_nbr)+1;
+  partition->part_size=(uint64_t)(le64(ntfs_header->sectors_nbr)+1)*ntfs_sector_size(ntfs_header);
   partition->part_type=(unsigned char)P_NTFS;
   if(backup)
   {
-    partition->boot_sector=partition->part_size-1;
-    partition->lba-=partition->part_size-1;
+    if(debug>1)
+      ecrit_rapport("NTFS part_offset=%llu, part_size=%llu, sector_size=%u\n",
+	  (long long unsigned)partition->part_offset, (long long unsigned)partition->part_size,
+	  disk_car->sector_size);
+    partition->boot_sector=partition->part_size/disk_car->sector_size-1;
+    partition->part_offset=partition->part_offset+disk_car->sector_size-partition->part_size;
+    if(debug>1)
+      ecrit_rapport("part_offset=%llu\n",(long long unsigned)partition->part_offset);
   }
   set_NTFS_info(disk_car,ntfs_header,partition,debug,dump_ind);
   return 0;
 }
 
-int set_NTFS_info(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_diskext *partition,const int debug, const int dump_ind)
+int set_NTFS_info(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind)
 {
   partition->name[0]='\0';
+  snprintf(partition->info,sizeof(partition->info),"NTFS");
   return ntfs_read_MFT(disk_car, partition, ntfs_header,0x60,debug,dump_ind);
 }
 
-int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_header, t_diskext *partition,const int debug, const int dump_ind)
+int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_header, t_partition *partition,const int debug, const int dump_ind)
 {
   const char*buffer=(const char*)ntfs_header;
   if(le16(ntfs_header->marker)==0xAA55)
@@ -100,24 +113,41 @@ int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_he
 	return 1;
       if((debug>0) || (dump_ind!=0))
       {
-	ecrit_rapport("NTFS at %u/%u/%u\n", LBA2cylinder(disk_car,partition->lba),LBA2head(disk_car,partition->lba),LBA2sector(disk_car,partition->lba));
+	ecrit_rapport("NTFS at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
 	if(dump_ind!=0)
-	  dump(stdscr,buffer,SECTOR_SIZE);
+	  dump(stdscr,buffer,DEFAULT_SECTOR_SIZE);
       }
+      if(ntfs_sector_size(ntfs_header)!=disk_car->sector_size)
+      {
+	aff_buffer(BUFFER_ADD,"test_NTFS: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",ntfs_sector_size(ntfs_header),disk_car->sector_size);
+	ecrit_rapport("test_NTFS: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",ntfs_sector_size(ntfs_header),disk_car->sector_size);
+	return 1;
+      }
+      if(le16(ntfs_header->secs_track)!=disk_car->CHS.sector)
+      {
+	aff_buffer(BUFFER_ADD,"check_NTFS: Incorrect number of sectors per track %u (NTFS) != %u (HD)\n",le16(ntfs_header->secs_track),disk_car->CHS.sector);
+	ecrit_rapport("sect/track %u (NTFS) != %u (HD)\n",le16(ntfs_header->secs_track),disk_car->CHS.sector);
+      }
+      if(le16(ntfs_header->heads)!=disk_car->CHS.head+1)
+      {
+	aff_buffer(BUFFER_ADD,"check_NTFS: Incorrect number of heads/cylinder %u (NTFS) != %u (HD)\n",le16(ntfs_header->heads),disk_car->CHS.head+1);
+	ecrit_rapport("heads/cylinder %u (NTFS) != %u (HD)\n",le16(ntfs_header->heads),disk_car->CHS.head+1);
+      }
+
       if(partition->part_size>0)
       {
-	dword part_size;
+	uint64_t part_size;
 	part_size=le64(ntfs_header->sectors_nbr)+1;
-	if(part_size>partition->part_size)
+
+	if(part_size*ntfs_sector_size(ntfs_header)>partition->part_size)
 	{
-	  ecrit_rapport("test_NTFS size boot_sector %lu > partition %lu\n",part_size,partition->part_size);
-	  aff_buffer(BUFFER_ADD,"test_NTFS size boot_sector %lu > partition %lu\n",part_size,partition->part_size);
+	  ecrit_rapport("test_NTFS size boot_sector %lu > partition %lu\n",(long unsigned)part_size,(long unsigned)(partition->part_size/disk_car->sector_size));
+	  aff_buffer(BUFFER_ADD,"test_NTFS size boot_sector %lu > partition %lu\n",(long unsigned)part_size,(long unsigned)(partition->part_size/disk_car->sector_size));
 	  return 1;
 	}
-	else
+	if((debug>0) && (part_size!=partition->part_size/disk_car->sector_size))
 	{
-	  if((debug>0) && (part_size!=partition->part_size))
-	    ecrit_rapport("test_NTFS size boot_sector %lu, partition %lu\n",part_size,partition->part_size);
+	    ecrit_rapport("test_NTFS size boot_sector %lu, partition %lu\n",(long unsigned)part_size,(long unsigned)(partition->part_size/disk_car->sector_size));
 	}
       }
       partition->upart_type=UP_NTFS;
@@ -128,7 +158,7 @@ int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_he
 }
 
 /* */
-static int ntfs_get_attr(const char *mft_record, const int my_type, t_diskext *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find)
+static int ntfs_get_attr(const char *mft_record, const int my_type, t_partition *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find)
 {
   const char *attr_record;
   /* Only check for magic DWORD here, fixup should have happened before */
@@ -145,7 +175,7 @@ static int ntfs_get_attr(const char *mft_record, const int my_type, t_diskext *p
   return ntfs_get_attr_aux(attr_record, my_type, partition, end, debug, dump_ind, file_name_to_find);
 }
 
-static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_diskext *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find)
+static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_partition *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find)
 {
   int attr_type;
   while(1)
@@ -257,7 +287,7 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_diske
 	    }
 	    return 1;
 	  case 0x90:	/* AT_INDEX_ROOT */
-	    return NTFS_GETU32(attr_list_entry+8);	// index_block_size
+	    return NTFS_GETU32(attr_list_entry+8);	/* index_block_size */
 	}
       }
     }
@@ -269,15 +299,15 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_diske
 	{
 	  case 0x80:	/* AT_DATA */
 	    {
-	      const __u8 *buf;
-	      __u8 b;                   /* Current byte offset in buf. */
-	      __s64 deltaxcn;           /* Change in [vl]cn. */
-	      __u16 mapping_pairs_offset;
-	      const __u8 *attr_end;     /* End of attribute. */
+	      const uint8_t *buf;
+	      uint8_t b;                   /* Current byte offset in buf. */
+	      int64_t deltaxcn;           /* Change in [vl]cn. */
+	      uint16_t mapping_pairs_offset;
+	      const uint8_t *attr_end;     /* End of attribute. */
 	      long lcn;
 	      mapping_pairs_offset=NTFS_GETU16(attr_record+32);
 	      buf=attr_record+mapping_pairs_offset;
-	      attr_end = (const __u8*)attr_record + attr_len;
+	      attr_end = (const uint8_t*)attr_record + attr_len;
 	      lcn = 0;
 	      /* return first element of the run_list */
 	      {
@@ -288,11 +318,11 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_diske
 		    ecrit_rapport("Attribut AT_DATA: bad size\n");
 		    return 0;
 		  }
-		  for (deltaxcn = (__s8)buf[b--]; b; b--)
+		  for (deltaxcn = (int8_t)buf[b--]; b; b--)
 		    deltaxcn = (deltaxcn << 8) + buf[b];
 		} else { /* The length entry is compulsory. */
 		  ecrit_rapport("Missing length entry in mapping pairs array.");
-		  deltaxcn = (__s64)-1;
+		  deltaxcn = (int64_t)-1;
 		}
 		if (deltaxcn < 0) {
 		  ecrit_rapport("Invalid length in mapping pairs array.");
@@ -306,14 +336,14 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_diske
 		else
 		{
 		  /* Get the lcn change which really can be negative. */
-		  __u8 b2 = *buf & 0xf;
+		  uint8_t b2 = *buf & 0xf;
 		  b = b2 + ((*buf >> 4) & 0xf);
 		  if (buf + b > attr_end)
 		  {
 		    ecrit_rapport("Attribut AT_DATA: bad size\n");
 		    return 0;
 		  }
-		  for (deltaxcn = (__s8)buf[b--]; b > b2; b--)
+		  for (deltaxcn = (int8_t)buf[b--]; b > b2; b--)
 		    deltaxcn = (deltaxcn << 8) + buf[b];
 		  /* Change the current lcn to it's new value. */
 		  lcn += deltaxcn;
@@ -338,33 +368,33 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_diske
   }
 }
 
-static int ntfs_read_MFT(t_param_disk *disk_car, t_diskext *partition, const struct ntfs_boot_sector*ntfs_header,const int my_type,const int debug, const int dump_ind)
+static int ntfs_read_MFT(t_param_disk *disk_car, t_partition *partition, const struct ntfs_boot_sector*ntfs_header,const int my_type,const int debug, const int dump_ind)
 {
   unsigned char *buffer;
   unsigned char *attr;
-  dword mft_pos;
+  uint64_t mft_pos;
   unsigned int mft_recordsize;
-  mft_pos=partition->lba+ntfs_header->reserved+ntfs_header->mft_lcn*ntfs_header->sectors_per_cluster;
+  mft_pos=partition->part_offset+(uint64_t)(ntfs_header->reserved+ntfs_header->mft_lcn*ntfs_header->sectors_per_cluster)*DEFAULT_SECTOR_SIZE;
   if(ntfs_header->clusters_per_index_record>0)
     mft_recordsize=ntfs_header->sectors_per_cluster*ntfs_header->clusters_per_index_record;
   else
     mft_recordsize=1<<(-ntfs_header->clusters_per_index_record);
-  buffer=(unsigned char *)MALLOC(mft_recordsize*SECTOR_SIZE);
+  buffer=(unsigned char *)MALLOC(mft_recordsize*DEFAULT_SECTOR_SIZE);
 #ifdef NTFS_DEBUG
   ecrit_rapport("NTFS cluster size = %u\n",ntfs_header->sectors_per_cluster);
   ecrit_rapport("NTFS MFT_record_size = %u\n",mft_recordsize);
   ecrit_rapport("NTFS MFT cluster = %lu\n",ntfs_header->mft_lcn);
 #endif
-  if(disk_car->read(disk_car,mft_recordsize, buffer, mft_pos)!=0)
+  if(disk_car->read(disk_car,mft_recordsize*DEFAULT_SECTOR_SIZE, buffer, mft_pos)!=0)
   {
     ecrit_rapport(msg_ROOT_CLUSTER_RERR);
     FREE(buffer);
     return 1;
   }
   attr=buffer;
-  while(attr+0x30<=(buffer+mft_recordsize*SECTOR_SIZE))
+  while(attr+0x30<=(buffer+mft_recordsize*DEFAULT_SECTOR_SIZE))
   {
-    int res=ntfs_get_attr(attr,my_type,partition,buffer+mft_recordsize*SECTOR_SIZE,debug,dump_ind,NULL);
+    int res=ntfs_get_attr(attr,my_type,partition,buffer+mft_recordsize*DEFAULT_SECTOR_SIZE,debug,dump_ind,NULL);
     if((res>0)|| (NTFS_GETU32(attr + 0x1C)<0x30))
     {
       FREE(buffer);
@@ -416,14 +446,14 @@ static int testdisk_ffs(int x)
   return r;
 }
 
-static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *partition, const int interface, const unsigned int cluster_size, const __u64 mft_lcn, const __u64 mftmirr_lcn, const __s32 mft_record_size, const __s32 index_block_size)
+static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_partition *partition, const int interface, const unsigned int cluster_size, const uint64_t mft_lcn, const uint64_t mftmirr_lcn, const int32_t mft_record_size, const int32_t index_block_size)
 {
-  unsigned char orgboot[SECTOR_SIZE];
-  unsigned char newboot[SECTOR_SIZE];
+  unsigned char orgboot[DEFAULT_SECTOR_SIZE];
+  unsigned char newboot[DEFAULT_SECTOR_SIZE];
   struct ntfs_boot_sector *org_ntfs_header=(struct ntfs_boot_sector *)&orgboot;
   struct ntfs_boot_sector *ntfs_header=(struct ntfs_boot_sector *)&newboot;
   int error=0;
-  if(disk_car->read(disk_car,1, &orgboot, partition->lba)!=0)
+  if(disk_car->read(disk_car,sizeof(orgboot), &orgboot, partition->part_offset)!=0)
   {
     ecrit_rapport(msg_CHKFAT_RERR); return 1;
   }
@@ -436,11 +466,11 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *parti
     ecrit_rapport("Bad extrapolation.\n");
     return 1;
   }
-  memcpy(&newboot,&orgboot,SECTOR_SIZE);
+  memcpy(&newboot,&orgboot,sizeof(newboot));
   memcpy(ntfs_header->system_id,"NTFS    ",8);
-  ntfs_header->sector_size[0]=SECTOR_SIZE & 0xFF;
-  ntfs_header->sector_size[1]=SECTOR_SIZE>>8;
-  ntfs_header->sectors_per_cluster=cluster_size/SECTOR_SIZE;
+  ntfs_header->sector_size[0]=disk_car->sector_size & 0xFF;
+  ntfs_header->sector_size[1]=disk_car->sector_size>>8;
+  ntfs_header->sectors_per_cluster=cluster_size/disk_car->sector_size;
   ntfs_header->reserved=0;
   ntfs_header->fats=0;
   ntfs_header->dir_entries[0]=0;
@@ -449,14 +479,14 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *parti
   ntfs_header->sectors[1]=0;
   ntfs_header->media=0xF8;
   ntfs_header->fat_length=0;
-  ntfs_header->secs_track=disk_car->CHS.sector;
-  ntfs_header->heads=disk_car->CHS.head+1;
+  ntfs_header->secs_track=le16(disk_car->CHS.sector);
+  ntfs_header->heads=le16(disk_car->CHS.head+1);
   if(partition->status==STATUS_LOG)
     ntfs_header->hidden=63;
   else
-    ntfs_header->hidden=partition->lba;
+    ntfs_header->hidden=partition->part_offset/disk_car->sector_size;
   ntfs_header->total_sect=0;
-  ntfs_header->sectors_nbr=partition->part_size-1;
+  ntfs_header->sectors_nbr=le64(partition->part_size/disk_car->sector_size-1);
   ntfs_header->mft_lcn=mft_lcn;
   ntfs_header->mftmirr_lcn=mftmirr_lcn;
   ntfs_header->clusters_per_mft_record=(mft_record_size >= cluster_size?mft_record_size / cluster_size:
@@ -471,16 +501,16 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *parti
   ntfs_header->reserved1[2]=0;
   /*
   {
-    __u32 *u;
-    __u32 checksum;
-    for (checksum = 0,u=(__u32*)ntfs_header; u < (__u32*)(&ntfs_header->checksum); u++)
+    uint32_t *u;
+    uint32_t checksum;
+    for (checksum = 0,u=(uint32_t*)ntfs_header; u < (uint32_t*)(&ntfs_header->checksum); u++)
       checksum += NTFS_GETU32(u);
     ntfs_header->checksum=checksum;
   }
   */
   ntfs_header->checksum=0;
   ntfs_header->marker=0xAA55;
-  if(memcmp(newboot,orgboot,SECTOR_SIZE))
+  if(memcmp(newboot,orgboot,sizeof(newboot)))
   {
     ecrit_rapport("             New / Current boot sector");
     dump_2ntfs_rapport(ntfs_header,org_ntfs_header);
@@ -505,11 +535,11 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *parti
     aff_copy(stdscr);
     wmove(stdscr,4,0);
     wdoprintf(stdscr,"%s",disk_car->description(disk_car));
-    mvwaddstr(stdscr,6,0,msg_PART_HEADER2);
-    wmove(stdscr,7,0);
+    mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
+    wmove(stdscr,6,0);
     aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
     wmove(stdscr,8,0);
-    if(memcmp(newboot,orgboot,SECTOR_SIZE))	/* Only compare the first sector */
+    if(memcmp(newboot,orgboot,sizeof(newboot)))	/* Only compare the first sector */
     {
       options="DWQ";
       dump_2ntfs_info(ntfs_header, org_ntfs_header);
@@ -533,7 +563,7 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *parti
       case 'D':
 	if(strchr(options,'D')!=NULL)
 	{
-	  dump2(stdscr,newboot,orgboot, SECTOR_SIZE);
+	  dump2(stdscr,newboot,orgboot, sizeof(newboot));
 	}
 	break;
     }
@@ -541,11 +571,11 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car,const t_diskext *parti
     {
       ecrit_rapport("Write new boot!\n");
       /* Write boot sector and backup boot sector */
-      if(disk_car->write(disk_car,1, &newboot, partition->lba))
+      if(disk_car->write(disk_car,sizeof(newboot), &newboot, partition->part_offset))
       {
 	display_message("Write error: Can't write new NTFS boot sector\n");
       }
-      if(disk_car->write(disk_car,1, &newboot, partition->lba+partition->part_size-1)!=0)
+      if(disk_car->write(disk_car,sizeof(newboot), &newboot, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
       {
 	display_message("Write error: Can't write new NTFS backup boot sector\n");
       }
@@ -596,10 +626,11 @@ int dump_ntfs_rapport(const struct ntfs_boot_sector *ntfs_header)
   return 0;
 }
 
-static int ntfs_find_mft(t_param_disk *disk_car, t_diskext *partition, const int debug, const int interface)
+
+int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind,const int interface)
 {
-  unsigned long int sector;
-  unsigned char buffer[8*SECTOR_SIZE];
+  uint64_t sector;
+  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
   int ind_stop=FALSE;
   int mft_sector=0;
   if(interface)
@@ -607,35 +638,36 @@ static int ntfs_find_mft(t_param_disk *disk_car, t_diskext *partition, const int
     aff_copy(stdscr);
     wmove(stdscr,4,0);
     wdoprintf(stdscr,"%s",disk_car->description(disk_car));
-    mvwaddstr(stdscr,6,0,msg_PART_HEADER2);
-    wmove(stdscr,7,0);
+    mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
+    wmove(stdscr,6,0);
     aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
   }
   /* try to find MFT Backup first */
-  for(sector=(partition->part_size/2-20>0?partition->part_size/2-20:1);(sector<partition->part_size)&&(sector<=partition->part_size/2+20)&&(ind_stop==FALSE);sector++)
+  for(sector=(partition->part_size/disk_car->sector_size/2-20>0?partition->part_size/disk_car->sector_size/2-20:1);(sector<partition->part_size/disk_car->sector_size)&&(sector<=partition->part_size/disk_car->sector_size/2+20)&&(ind_stop==FALSE);sector++)
   {
-    if(disk_car->read(disk_car,2, &buffer, partition->lba+sector)!=0)
-    { return 1; }
-    if(memcmp(buffer,"FILE",4)==0 && (NTFS_GETU16(buffer+ 0x14)%8==0) && (NTFS_GETU16(buffer+ 0x14)>=42)
-      &&(NTFS_GETU16(buffer+22)==1))	/* MFT_RECORD_IN_USE */
+    if(disk_car->read(disk_car,2*DEFAULT_SECTOR_SIZE, &buffer, partition->part_offset+sector*(uint64_t)disk_car->sector_size)==0)
     {
-      int res;
-      res=ntfs_get_attr(buffer,0x30,partition,buffer+2*SECTOR_SIZE,debug,0,"$MFT");
-      if(res==1)
+      if(memcmp(buffer,"FILE",4)==0 && (NTFS_GETU16(buffer+ 0x14)%8==0) && (NTFS_GETU16(buffer+ 0x14)>=42)
+	  &&(NTFS_GETU16(buffer+22)==1))	/* MFT_RECORD_IN_USE */
       {
-	mft_sector=sector;
-	ecrit_rapport("mft at %ld, seq=%u, main=%u res=%d\n",sector,NTFS_GETU8(buffer+0x10),NTFS_GETU32(buffer+0x20),res);
-	ind_stop=TRUE;
+	int res;
+	res=ntfs_get_attr(buffer,0x30,partition,buffer+2*DEFAULT_SECTOR_SIZE,debug,0,"$MFT");
+	if(res==1)
+	{
+	  mft_sector=sector;
+	  ecrit_rapport("mft at %lu, seq=%u, main=%u res=%d\n",(long unsigned)sector,NTFS_GETU8(buffer+0x10),(unsigned int)NTFS_GETU32(buffer+0x20),res);
+	  ind_stop=TRUE;
+	}
       }
     }
   }
-  for(sector=1;(sector<partition->part_size)&&(ind_stop==FALSE);sector++)
+  for(sector=1;(sector<partition->part_size/disk_car->sector_size)&&(ind_stop==FALSE);sector++)
   {
     if((interface!=0) &&(sector%200)==0)
     {
       wmove(stdscr,9,0);
       wclrtoeol(stdscr);
-      wdoprintf(stdscr,"Search mft %10lu/%lu",sector,partition->part_size);
+      wdoprintf(stdscr,"Search mft %10lu/%lu",(long unsigned)sector,(long unsigned)(partition->part_size/disk_car->sector_size));
       wrefresh(stdscr);
       switch(wgetch_nodelay(stdscr))
       {
@@ -651,34 +683,38 @@ static int ntfs_find_mft(t_param_disk *disk_car, t_diskext *partition, const int
 	  break;
       }
     }
-    if(disk_car->read(disk_car,2, &buffer, partition->lba+sector)!=0)
-    { return 1; }
-    if(memcmp(buffer,"FILE",4)==0 && (NTFS_GETU16(buffer+ 0x14)%8==0) && (NTFS_GETU16(buffer+ 0x14)>=42))
+    if(disk_car->read(disk_car,2*DEFAULT_SECTOR_SIZE, &buffer, partition->part_offset+sector*(uint64_t)disk_car->sector_size)==0)
     {
-      int res;
-      res=ntfs_get_attr(buffer,0x30,partition,buffer+2*SECTOR_SIZE,debug,0,"$MFT");
-      if(res==1)
+      if(memcmp(buffer,"FILE",4)==0 && (NTFS_GETU16(buffer+ 0x14)%8==0) && (NTFS_GETU16(buffer+ 0x14)>=42))
       {
-	mft_sector=sector;
-	ecrit_rapport("mft at %ld, seq=%u, main=%u res=%d\n",sector,NTFS_GETU8(buffer+0x10),NTFS_GETU32(buffer+0x20),res);
-	ind_stop=TRUE;
+	int res;
+	res=ntfs_get_attr(buffer,0x30,partition,buffer+2*DEFAULT_SECTOR_SIZE,debug,0,"$MFT");
+	if(res==1)
+	{
+	  mft_sector=sector;
+	  ecrit_rapport("mft at %lu, seq=%u, main=%u res=%d\n",(long unsigned)sector,NTFS_GETU8(buffer+0x10),(unsigned int)NTFS_GETU32(buffer+0x20),res);
+	  ind_stop=TRUE;
+	}
       }
     }
   }
   if(mft_sector>0)
   {
-    __u64 mft_lcn;
-    __u64 mftmirr_lcn;
+    uint64_t mft_lcn;
+    uint64_t mftmirr_lcn;
     unsigned int sectors_per_cluster=0;
     const char *attr=buffer;
     unsigned int index_block_size=4096;
     unsigned int mft_record_size=1024;
-    if(disk_car->read(disk_car,8, &buffer, partition->lba+mft_sector)!=0)
-    { return 1; }
-    mft_lcn=ntfs_get_attr(attr,0x80,partition,buffer+8*SECTOR_SIZE,debug,0,NULL);
+    if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)mft_sector*disk_car->sector_size)!=0)
+    {
+      ecrit_rapport("NTFS: Can't read mft_sector\n");
+      return 1;
+    }
+    mft_lcn=ntfs_get_attr(attr,0x80,partition,buffer+8*DEFAULT_SECTOR_SIZE,debug,0,NULL);
     mft_record_size=NTFS_GETU32(attr + 0x1C);
     attr+= NTFS_GETU32(attr + 0x1C);
-    mftmirr_lcn=ntfs_get_attr(attr,0x80,partition,buffer+8*SECTOR_SIZE,debug,0,NULL);
+    mftmirr_lcn=ntfs_get_attr(attr,0x80,partition,buffer+8*DEFAULT_SECTOR_SIZE,debug,0,NULL);
     if(mft_lcn>0 && mft_sector%mft_lcn==0)
     {
       sectors_per_cluster=mft_sector/mft_lcn;
@@ -690,8 +726,11 @@ static int ntfs_find_mft(t_param_disk *disk_car, t_diskext *partition, const int
     if(sectors_per_cluster>0 && mft_record_size>0)
     {
       /* Read "root directory" in MFT */
-      if(disk_car->read(disk_car,mft_record_size/SECTOR_SIZE, &buffer, partition->lba+mft_lcn*sectors_per_cluster+5*mft_record_size/SECTOR_SIZE)!=0)
-      { return 1; }
+      if(disk_car->read(disk_car,mft_record_size, &buffer, partition->part_offset+(uint64_t)mft_lcn*sectors_per_cluster*disk_car->sector_size+5*(uint64_t)mft_record_size)!=0)
+      {
+	ecrit_rapport("Can't read \"root directory\" in MFT\n");
+	return 1;
+      }
       index_block_size=ntfs_get_attr(buffer,0x90,partition,buffer+mft_record_size,debug,0,NULL);
       if(index_block_size%512!=0)
 	index_block_size=4096;
@@ -701,13 +740,7 @@ static int ntfs_find_mft(t_param_disk *disk_car, t_diskext *partition, const int
     ecrit_rapport("ntfs_find_mft: mftmirr_lcn         %lu\n",(long unsigned int)mftmirr_lcn);
     ecrit_rapport("ntfs_find_mft: mft_record_size     %u\n",mft_record_size);
     ecrit_rapport("ntfs_find_mft: index_block_size    %u\n",index_block_size);
-    create_ntfs_boot_sector(disk_car,partition, interface, sectors_per_cluster*SECTOR_SIZE, mft_lcn, mftmirr_lcn, mft_record_size, index_block_size);
+    create_ntfs_boot_sector(disk_car,partition, interface, sectors_per_cluster*disk_car->sector_size, mft_lcn, mftmirr_lcn, mft_record_size, index_block_size);
   }
   return 0;
 }
-
-int rebuild_NTFS_BS(t_param_disk *disk_car, t_diskext *partition, const int debug, const int dump_ind,const int interface)
-{
-  ntfs_find_mft(disk_car,partition,debug,interface);
-  return 0;
-}
