@@ -29,12 +29,20 @@
  
 #ifdef HAVE_LIBNTFS
 
+#include <sys/stat.h>
+
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 #include <time.h>
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include <errno.h>
-#include <stdint.h>
 #include <sys/param.h>
 #ifdef HAVE_MACHINE_ENDIAN_H
 #include <machine/endian.h>
@@ -47,7 +55,7 @@
 #define NTFS_TIME_OFFSET ((s64)(369 * 365 + 89) * 24 * 3600 * 10000000)
 #include "types.h"
 #include "common.h"
-#include "intrface.h"
+#include "intrf.h"
 #include "ntfs.h"
 #include "dir.h"
 struct list_dir_struct {
@@ -62,7 +70,7 @@ struct list_dir_struct {
  * This allows the caller to read directories into their application or
  * to have different dirent layouts depending on the binary type.
  */
-typedef int (*ntfs_filldir_t)(void *dirent, const uchar_t *name,
+typedef int (*ntfs_filldir_t)(void *dirent, const ntfschar *name,
                 const int name_len, const int name_type, const s64 pos,
                 const MFT_REF mref, const unsigned dt_type);
 
@@ -71,9 +79,9 @@ extern struct ntfs_device_operations ntfs_device_testdisk_io_ops;
 extern int ntfs_readdir(ntfs_inode *dir_ni, s64 *pos,
                 void *dirent, ntfs_filldir_t filldir);
 extern u64 ntfs_inode_lookup_by_name(ntfs_inode *dir_ni,
-                const uchar_t *uname, const int uname_len);
+                const ntfschar *uname, const int uname_len);
 time_t ntfs2utc (s64 ntfstime);
-int list_entry(  struct list_dir_struct *ls, const uchar_t *name, 
+int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type);
 ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, const char *pathname);
@@ -97,7 +105,7 @@ time_t ntfs2utc (s64 ntfstime)
  * list_entry
  * FIXME: Should we print errors as we go along? (AIA)
  */
-int list_entry(  struct list_dir_struct *ls, const uchar_t *name, 
+int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type)
 {
@@ -173,7 +181,7 @@ int list_entry(  struct list_dir_struct *ls, const uchar_t *name,
       new_file->filestat.st_gid=0;
       new_file->filestat.st_rdev=0;
       new_file->filestat.st_size=filesize;
-      new_file->filestat.st_blksize=SECTOR_SIZE;
+      new_file->filestat.st_blksize=DEFAULT_SECTOR_SIZE;
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
       if(new_file->filestat.st_blksize!=0)
       {
@@ -254,7 +262,7 @@ ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, cons
 	char *p, *q;
 	ntfs_inode *ni;
 	ntfs_inode *result  = NULL;
-	uchar_t    *unicode = NULL;
+	ntfschar    *unicode = NULL;
 	char       *ascii   = NULL;
 
 	if (!vol || !pathname) {
@@ -355,8 +363,8 @@ ATTR_RECORD * find_attribute (const ATTR_TYPES type, ntfs_attr_search_ctx *ctx)
         return ctx->attr;
 }
 
-t_file_data *ntfs_dir(t_param_disk *disk_car, const t_diskext *partition, t_dir_data *dir_data, const unsigned long int cluster);
-t_file_data *ntfs_dir(t_param_disk *disk_car, const t_diskext *partition, t_dir_data *dir_data, const unsigned long int cluster)
+t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
+t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
 {
   ntfs_inode *ni;
   s64 pos;
@@ -393,27 +401,14 @@ t_file_data *ntfs_dir(t_param_disk *disk_car, const t_diskext *partition, t_dir_
   return ls->dir_list;
 }
 
-/**
- * main - Begin here
- *
- * Start from here.
- *
- * Return:  0  Success, the program worked
- *	    1  Error, parsing mount options failed
- *	    2  Error, mount attempt failed
- *	    3  Error, failed to open root directory
- *	    4  Error, failed to open directory in search path
- */
-int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_diskext *partition, const int debug)
+int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
 {
   t_my_data my_data;
   static struct ntfs_device *dev;
   ntfs_volume *vol=NULL;
   aff_buffer(BUFFER_RESET,"Q");
   wmove(window,4,0);
-  aff_part(window,AFF_PART_NL,disk_car,partition);
-  ecrit_rapport("\n");
-  aff_part_rapport(disk_car,partition);
+  aff_part(window,AFF_PART_ORDER,disk_car,partition);
   my_data.partition=partition;
   my_data.disk_car=disk_car;
   my_data.offset=0;
@@ -425,21 +420,13 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_diskext *partitio
     vol = ntfs_device_mount(dev, MS_RDONLY);
   }
   if (!vol) {
-    ecrit_rapport("Couldn't mount device %s\n", strerror (errno));
+    aff_part_rapport(disk_car,partition);
+    aff_buffer(BUFFER_ADD,"Can't open NTFS filesystem: %s\n",strerror (errno));
+    aff_buffer(BUFFER_DISPLAY,"Q",window);
     return 2;
   }
-  else
-  {
-    if(debug>1)
-    {
-      ecrit_rapport("dir_partition_ntfs: ntfs_device_mount ok\n");
-    }
-  }
-
   if (vol->flags & VOLUME_IS_DIRTY) {
     ecrit_rapport("Volume is dirty.\n");
-/*   ntfs_umount (vol, FALSE); */
-/*   return 2; */
   }
   {
     struct list_dir_struct ls;
@@ -460,13 +447,13 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_diskext *partitio
 #else
 #include "types.h"
 #include "common.h"
-#include "intrface.h"
+#include "intrf.h"
 
-int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_diskext *partition)
+int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition)
 {
   aff_buffer(BUFFER_RESET,"Q");
-  wmove(stdscr,5,0);
-  aff_part(window,AFF_PART_NONL,disk_car,partition);
+  wmove(stdscr,4,0);
+  aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with ntfsprogs library");
   aff_buffer(BUFFER_DISPLAY,"Q",window);
