@@ -26,11 +26,21 @@
  
 #ifdef HAVE_LIBREISERFS
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
 #include <time.h>
 #include "dal/dal.h"
 #ifdef HAVE_DAL_FILE_DAL_H
@@ -42,7 +52,7 @@
 #include "reiserfs/reiserfs.h"
 #include "types.h"
 #include "common.h"
-#include "intrface.h"
+#include "intrf.h"
 #include "rfs.h"
 #include "dir.h"
 
@@ -105,38 +115,39 @@ char *dal_error(dal_t *dal) {
 }
 
 static int file_read(dal_t *dal, void *buff, blk_t block, blk_t count) {
-  dword off, blocklen;
+  uint64_t off;
+  unsigned int blocklen;
   t_my_data *my_data=(t_my_data*)dal->data;
 /* ecrit_rapport("reiser file_read(dal=%p,buff=%p,block=%ld, count=%ld)\n",dal,buff,block,count); */
   if (!dal || !buff)
     return 0;
 #ifdef HAVE_DAL_T_BLOCK_SIZE
-  off = (dword)block * ((dword)dal->block_size/SECTOR_SIZE);
-  blocklen = (dword)count * ((dword)dal->block_size/SECTOR_SIZE);
+  off = (uint64_t)block * (uint64_t)dal->block_size;
+  blocklen = count * dal->block_size;
 #else
-  off = (dword)block * ((dword)dal->blocksize/SECTOR_SIZE);
-  blocklen = (dword)count * ((dword)dal->blocksize/SECTOR_SIZE);
+  off = (uint64_t)block * (uint64_t)dal->blocksize;
+  blocklen = count * dal->blocksize;
 #endif
 /* ecrit_rapport("blocklen=%ld\n",blocklen); */
-  if(my_data->disk_car->read(my_data->disk_car,blocklen,buff,my_data->partition->lba+off))
+  if(my_data->disk_car->read(my_data->disk_car,blocklen,buff,my_data->partition->part_offset+off))
     return 0;
   return 1;
 }
 
 static int file_write(dal_t *dal, void *buff, blk_t block, blk_t count)
 {
-  dword off, blocklen;
-/* ecrit_rapport("reiser file_write\n"); */
+  uint64_t off, blocklen;
   if (!dal || !buff)
     return 0;
 #ifdef HAVE_DAL_T_BLOCK_SIZE
-  off = (dword)block * ((dword)dal->block_size/SECTOR_SIZE);
-  blocklen = (dword)count * ((dword)dal->block_size/SECTOR_SIZE);
+  off = (uint64_t)block * (uint64_t)dal->block_size;
+  blocklen = (uint64_t)count * (uint64_t)dal->block_size;
 #else
-  off = (dword)block * ((dword)dal->blocksize/SECTOR_SIZE);
-  blocklen = (dword)count * ((dword)dal->blocksize/SECTOR_SIZE);
+  off = (uint64_t)block * (uint64_t)dal->blocksize;
+  blocklen = (uint64_t)count * (uint64_t)dal->blocksize;
 #endif
-/* if(my_data->disk_car->write(my_data->disk_car,blocklen,buff,my_data->partition->lba+off)) */
+  ecrit_rapport("reiser file_write not implemented\n");
+/* if(my_data->disk_car->write(my_data->disk_car,blocklen,buff,my_data->partition->part_offset+off)) */
     return 0;
 /* return 1; */
 }
@@ -160,7 +171,12 @@ static int file_equals(dal_t *dal1, dal_t *dal2)
 /* ecrit_rapport("reiser file_equals\n"); */
   if (!dal1 || !dal2)
     return 0;
-  return !strcmp((char *)dal1->data, (char *)dal2->data);
+/* return !strcmp((char *)dal1->data, (char *)dal2->data); */
+  {
+    t_my_data *data1=(t_my_data*)dal1->data;
+    t_my_data *data2=(t_my_data*)dal2->data;
+    return (data1->disk_car==data2->disk_car) && (data1->partition==data2->partition) && (data1->offset==data2->offset);
+  }
 }
 
 #ifdef HAVE_STRUCT_DAL_OPS_DEV
@@ -188,15 +204,13 @@ static blk_t   file_len(dal_t *dal) {
 #else
 static count_t file_len(dal_t *dal) {
 #endif
-  dword max_off = 0;
   t_my_data *my_data=(t_my_data*)dal->data;
 /* ecrit_rapport("reiser file_len\n"); */
   if (!dal) return 0;
-  max_off=my_data->partition->part_size*SECTOR_SIZE;
 #ifdef HAVE_DAL_T_BLOCK_SIZE
-  return max_off / dal->block_size;
+  return my_data->partition->part_size / dal->block_size;
 #else
-  return max_off / dal->blocksize;
+  return my_data->partition->part_size / dal->blocksize;
 #endif
 }
 
@@ -275,7 +289,7 @@ dal_t *dal_open(struct dal_ops *myops, unsigned blocksize, int flags, void *data
 }
 
 #ifdef HAVE_STRUCT_DAL_OPS_DEV
-int dal_set_block_size(dal_t *dal, unsigned blocksize) {
+int dal_set_block_size(dal_t *dal, size_t blocksize) {
 
 	if (!dal) return 0;
 	
@@ -390,8 +404,8 @@ blk_t dal_len(dal_t *dal) {
 	return 0;
 }
 
-t_file_data *reiser_dir(t_param_disk *disk_car, const t_diskext *partition, t_dir_data *dir_data, const unsigned long int cluster);
-t_file_data *reiser_dir(t_param_disk *disk_car, const t_diskext *partition, t_dir_data *dir_data, const unsigned long int cluster)
+t_file_data *reiser_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
+t_file_data *reiser_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
 {
   struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
   reiserfs_dir_t *dir;
@@ -416,7 +430,7 @@ t_file_data *reiser_dir(t_param_disk *disk_car, const t_diskext *partition, t_di
       t_file_data *new_file=MALLOC(sizeof(*new_file));
       thislen=(MAX_NAME_LEN(DEFAULT_BLOCK_SIZE)<DIR_NAME_LEN?MAX_NAME_LEN(DEFAULT_BLOCK_SIZE):DIR_NAME_LEN);
       memcpy(new_file->name,entry.de_name,thislen);
-      new_file->name[thislen]='\0';
+      new_file->name[thislen-1]='\0';
 
       new_file->prev=ls->current_file;
       new_file->next=NULL;
@@ -435,7 +449,7 @@ t_file_data *reiser_dir(t_param_disk *disk_car, const t_diskext *partition, t_di
 }
 
 
-int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_diskext *partition, const int debug)
+int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
 {
   dal_t *dal;
   reiserfs_fs_t *fs;
@@ -446,11 +460,10 @@ int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_diskext *partit
 
   aff_buffer(BUFFER_RESET,"Q");
   wmove(window,4,0);
-  aff_part(window,AFF_PART_NL,disk_car,partition);
-  ecrit_rapport("\n");
-  aff_part_rapport(disk_car,partition);
+  aff_part(window,AFF_PART_ORDER,disk_car,partition);
   my_data.partition=partition;
   my_data.disk_car=disk_car;
+  my_data.offset=0;
 #ifdef HAVE_STRUCT_DAL_OPS_DEV
   dal=dal_open(&ops, "",DEFAULT_BLOCK_SIZE, O_RDONLY, &my_data);
 #else
@@ -458,6 +471,7 @@ int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_diskext *partit
 #endif
   if (!dal)
   {
+    aff_part_rapport(disk_car,partition);
     ecrit_rapport("Couldn't open device\n");
     aff_buffer(BUFFER_ADD,"Couldn't open device\n");
     goto dir_partition_reiser_error;
@@ -495,19 +509,20 @@ error_free_dal:
   file_close(dal);
 dir_partition_reiser_error:
   aff_buffer(BUFFER_DISPLAY,"Q",window);
-  return 0xff;    
+  return 0xff;
 }
 
 #else
 #include "types.h"
 #include "common.h"
-#include "intrface.h"
+#include "intrf.h"
+#include "rfs.h"
 
-int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_diskext *partition)
+int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
 {
   aff_buffer(BUFFER_RESET,"Q");
-  wmove(stdscr,5,0);
-  aff_part(window,AFF_PART_NONL,disk_car,partition);
+  wmove(stdscr,4,0);
+  aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with progsreiserfs library");
   aff_buffer(BUFFER_DISPLAY,"Q",window);
