@@ -23,23 +23,44 @@
 #include <config.h>
 #endif
  
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "swap.h"
 #include "fnctdsk.h"
+static int set_Linux_SWAP_info(const union swap_header *swap_header,t_partition *partition);
 
-int check_Linux_SWAP(t_param_disk *disk_car,t_diskext *partition,const int debug)
+int check_Linux_SWAP(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
-  unsigned char buffer[8*SECTOR_SIZE];
-  if(disk_car->read(disk_car,8, &buffer, partition->lba)!=0)
+  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
   { return 1; }
   if(test_Linux_SWAP(disk_car,(union swap_header*)&buffer,partition,debug,0)!=0)
     return 1;
+  set_Linux_SWAP_info((union swap_header*)&buffer,partition);
   return 0;
 }
 
-int test_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header,t_diskext *partition,const int debug, const int dump_ind)
+static int set_Linux_SWAP_info(const union swap_header *swap_header,t_partition *partition)
+{
+  switch(partition->upart_type)
+  {
+    case UP_LINSWAP:
+      snprintf(partition->info,sizeof(partition->info),"SWAP version %u",swap_header->info.version);
+      break;
+    case UP_LINSWAP2:
+      snprintf(partition->info,sizeof(partition->info),"SWAP2 version %u",swap_header->info.version);
+      break;
+    default:
+      partition->info[0]='\0';
+      break;
+  }
+  return 0;
+}
+
+int test_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header,t_partition *partition,const int debug, const int dump_ind)
 {
   if(memcmp(swap_header->magic.magic,"SWAP-SPACE",10)==0)
   {
@@ -54,10 +75,11 @@ int test_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header
   return 1;
 }
 
-int recover_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header,t_diskext *partition,const int debug, const int dump_ind)
+int recover_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header,t_partition *partition,const int debug, const int dump_ind)
 {
   if(test_Linux_SWAP(disk_car,swap_header,partition,debug,dump_ind)!=0)
     return 1;
+  set_Linux_SWAP_info(swap_header,partition);
   partition->part_type=(unsigned char)P_LINSWAP;
   switch(partition->upart_type)
   {
@@ -70,15 +92,11 @@ int recover_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_hea
 	for(j=7;j>=0;j--)
 	  if((swap_header->magic.reserved[i]&(1<<j))!=(char)0)
 	    break;
-	partition->part_size=(dword)((8*i+j+1)*(PAGE_SIZE/SECTOR_SIZE));
-	if(debug>1)
-	  ecrit_rapport("SWAP version %u\n",swap_header->info.version);
+	partition->part_size=(uint64_t)((8*i+j+1)*PAGE_SIZE);
       }
       break;
     case UP_LINSWAP2:
-      partition->part_size=(swap_header->info.last_page - 1)*(PAGE_SIZE/SECTOR_SIZE);
-      if(debug>1)
-	ecrit_rapport("SWAP2 version %u\n",swap_header->info.version);
+      partition->part_size=(uint64_t)(swap_header->info.last_page - 1)*PAGE_SIZE;
       break;
     default:
       return 1;
